;; RegWin.scm
;; (make-reg-win title fieldsname regfn)
;;
;; This creates a window with a specified title, that prompts the user for
;; a set of values for named fields (the programmer provides names and default values for the fields)
;; When the user hits the "go" button, the regfn is called on a tagger "t"
;;
;; Example of use:
;; (define R (make-reg-win "Register" '((a "") (b 5) (c "yes")) (lambda L (display L) (newline))))
;; (.show (R 'get 'window))
;;
;;
;; Note: this (requires jlib.Swing)
;;

(jlib.Swing.load)

(define (make-reg-win title fieldnames regfn)
  (define t0 (maketagger))
  (define t (lambda R
    (case (length R)
      ((1) (tryCatch (t0 (first R)) (lambda(e) (display (list "unknown tag" (first R)))(newline) #null)))
      ((2) (apply t0 R))
      (else (display (list "too many arguments to the tagger: " R))(newline) #null))))

  (define w (window "Registration"
     (border
        (north (label title))
        (center 
           (apply table 
             `( ,(length fieldnames) 2
                ,@(apply append
                    (map 
                      (lambda (x) 
                         (list (label (string-append (first x))) 
                                (t (first x) 
                                   (if (pair? (second x))
                                       (apply choice (map .toString (second x)))
                                       (textfield (string-append (second x)) 20)))))
                      fieldnames)))))
        (south (button "go" (action (lambda(e) (apply regfn (map readstring (map t (map first fieldnames)))))))))))
  (.pack w)
  (lambda (cmd . args)
    (case cmd
      ((get) 
       (case (first args)
         ((window) w)
         ((tagger) t)))))
)
  