;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Generic File Menu
;;
;; this adds the standard New,Open,Save,SaveAs,Quit menu items to
;; a file menu for a given window "win" and a given controller:
;;  (control 'string-to-save) generates a string to be saved
;;  (control 'new) is called when the user selects the "new" menu item
;;  (control 'open FILE STRING) does something to the string that is read from the disk
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define (addFileMenuItems FileMenu win control)
   (let* (
    (currentfile #null)
    (FileDialogLoad 
        (tryCatch (java.awt.FileDialog. win "File Loader" java.awt.FileDialog.LOAD$)
                  (lambda(e) (display e) (newline) #null)))
    (FileDialogSave  
        (tryCatch (java.awt.FileDialog. win "File Saver" java.awt.FileDialog.SAVE$)
                  (lambda(e) (display e) (newline)  #null)))
    (set-accelerator (lambda (K)
        (lambda(x)
              (.setAccelerator x (javax.swing.KeyStroke.getKeyStroke
                       K
                       java.awt.Event.ALT_MASK$)))))
    (set-mnemonic (lambda(K)
              (lambda(x) (.setMnemonic x K))))
    (readStreamIntoString  (lambda (stream) 
     (begin 
      (define reader  
          (java.io.BufferedReader. stream))
      (define (readloop sb) 
        (let ((x (.readLine reader))) 
          (if (equal? #null x) sb 
              (readloop  
                 (.append  
                     (.append sb x)  
                     "\n"))))) 
      (.toString 
         (readloop 
            (java.lang.StringBuffer. 8192))))))

    (save_from_dialog_data (lambda()
         (display "saving file") (newline)
         (tryCatch 
                  (begin
                   ;(.show FileDialogSave) 
                   (let ((chars (java.io.BufferedReader. (java.io.StringReader. 
                                   (control 'string-to-save))))
                         (outfile (java.io.PrintWriter. (java.io.FileWriter. 
                              (java.io.File.
                                 (.getDirectory FileDialogSave)
                                 (.getFile FileDialogSave))))))
                     (display (list "writing " chars " to " outfile))(newline)
                    ;; here we need to read lines from chars and write them to outfile...
                    ;(.write outfile chars 0 (.length chars))
                    (let loop ((x (.readLine chars)))
                         (if (equal? #null x) 
                             (begin 
                                 (.close outfile) (.close chars))
                             (begin 
                                (display x) (newline)
                                (.println outfile x)
                                (loop (.readLine chars)))))))

                 (lambda(err) (display (list "error" err))(newline) #t))))

      )
      (for-each (lambda(x) (.add FileMenu x))
       (list
         (menuitem "new" 
           (action (lambda(e) 
             (control 'new) 
             (.setFile FileDialogSave #null)
             (set! current-file #null))))
         (menuitem "open" 
           (set-mnemonic #'O')
           (action (lambda(e) 
             (tryCatch 
               (begin 
                (display "showing dialog")(newline)
                (.show FileDialogLoad) 
                (display "setting file")(newline)
                (set! currentfile (.getFile FileDialogLoad))
                (.setFile FileDialogSave (.getFile FileDialogLoad))
                (display "reading file")(newline)
                 (let* (
                     (infile 
                         (java.io.File. 
                             (.getDirectory FileDialogLoad) 
                             (.getFile FileDialogLoad)))
                     (infile-reader 
                         (java.io.FileReader. infile)))
                     (control 'open 
                           infile 
                           (string->expr (readStreamIntoString infile-reader)))
                     (.close infile-reader)))
               (lambda(err) (display (list "ERROR" err))(newline) #t)
               ))))
       
            (menuitem "save"     ;(lambda(x)  (.setVisible x #f))
              (set-mnemonic #'S')
              (action
               (lambda(e)
                 (if (or (equal? #null (.getFile FileDialogSave))
                         (equal? "" (.getFile FileDialogSave)))
                     (.show FileDialogSave))
                 (if (and (not (equal? (.getFile FileDialogSave) #null))
                        (equal? javax.swing.JOptionPane.OK_OPTION$
                          (javax.swing.JOptionPane.showConfirmDialog win
                          (string-append "Save to local file:\n     " 
                            ;(.getDirectory FileDialogSave)  
                            (.getFile FileDialogSave) 
                            " ??? "))))
                    (save_from_dialog_data)
                    (javax.swing.JOptionPane.showMessageDialog win 
                         "File not saved"))
               )))
  
           (menuitem "save as"
              (set-mnemonic #'A')
                (action (lambda(e)
                  (.show FileDialogSave) 
                  (save_from_dialog_data))))

         (menuitem "close" 
          (set-accelerator java.awt.event.KeyEvent.VK_Q$)
          (action 
           (lambda(e)
             (control 'quit)
             (.hide win))))
        ))))



