;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; NetFileChooser.scm
;;
;;
;; This creates a filechooser with open/save/saveas options for
;; the local disk as well as for network filesystems
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(jlib.Swing.load)

(define (run-nfc)
  (define win (window "demo"))
  (define FS (list "tat.cs.brandeis.edu" 53642))
  (define N (make-servent 0))
  (define M (netfilechooser "guest" "thanks" win N 
      (lambda R (display (list "OK" R)) (newline))))
  (display (list "net address:" (N 'get 'address))) (newline)
  (.show (M 'get 'win))
   M
)

(define (netfilechooser user password parent N currentFileserver OKaction)
  (define t (maketagger))
  (define bgcolor (color 250 220 190))
  (define buttoncolor (color 250 240 230))
  (define warning 
     (let ()
       (define ta (textarea 10 40 (HelveticaBold 18)))
       (define w (window "WARNING" 
         (border 
            (north 
                (label "WARNING" (HelveticaBold 24) white))
            (center ta)
            (south 
                (row 'none 'center bgcolor (button "OK" (action (lambda(e) (.hide w)))))))))
       (.setLineWrap ta #t)
       (lambda(msg)
           (.setText ta msg)
           (.pack w)
           (.show w) w)))

  ;; STATE VARIABLES
  (define currentpath (list ""))
  (define current-netservers ())

  (define dir-vec (java.util.Vector.))
  (define file-vec(java.util.Vector.))

  (define (get-path) (readstring (t "directory")))
  (define (get-filename) 
       (string-append (readstring (t "directory")) "/" (readstring (t "file"))))
  (define (update-dir)
       (jlistwrite (t "dirlist") ())
       (jlistwrite (t "filelist") ())
       (.invalidate (t "dirlist"))
       (.invalidate (t "filelist"))
       (.validate (t "sp0"))
       (.validate (t "sp1"))
       (.validate (t "sp2"))

       (N 'send 
             (readexpr (t "current netserver")) 
            'dir 
            (list (readstring (t "name")) (readstring (t "password"))(N 'get 'address))
            (get-path))
       (writestring (t "file") ""))

  (define netserverwin (window "netserver selection"
    (border
          (north 
           (col bgcolor 'center 'none
              (row 'center 'horizontal bgcolor (label "Network File Chooser" (HelveticaBold 32) bgcolor))
              (table 3 2 bgcolor
                (label "current netserver") 
                        (t "current netserver" 
                          (textfield (jscheme.REPL.printToString currentFileserver #t)
                            20 bgcolor (lambda(this) #t )))
                (label "netservers") 
                        (t "netserverlist" (jlist (lambda(x)
                                 (set! currentpath (list ""))
                                 (writestring (t "directory") "/")
                                 (writestring (t "file") "")
                                 (writestring (t "preview") "")
                                 (writeexpr (t "current netserver") x) (update-dir))))
                (label "username") (t "name" (textfield user 30))
                (label "password") (t "password" (javax.swing.JPasswordField. password))
               ))))))

  (define win 
;      (dialog parent #t "Net File Chooser"
      (window "Net File Chooser"
       (menubar
         (menu "File"
              (menuitem "quit" (action (lambda (e) (.hide win))))
              (menuitem "new folder" buttoncolor (action (lambda(e)
                 (writestring (t "new directory") "")
                 (.show mkdirwin)
                 (let ((newdir (readstring (t "new directory"))))
                   (if (not (equal? "" newdir))
                       (begin
                          (N 'send (readexpr (t "current netserver")) 'mkdir 
                             (list (readstring (t "name")) (readstring (t "password")) (N 'get 'address))
                              (string-append (get-path) "/" newdir))
                          (update-dir)))))))
              (menuitem "remove folder" buttoncolor (action (lambda(e)
                 (let ((dirs (.getSize(.getModel (t "dirlist"))))
                       (files (.getSize (.getModel (t "filelist")))))
                   (if (and (not (null? (rest currentpath)))
                            (or (> dirs 1) (> files 0)))
                       (warning (string-append "You must delete all files and subfolders of\n     " (get-path)
                                     "\nbefore you can delete it."))
                       (begin
                           (N 'send (readexpr (t "current netserver")) 'delete 
                                (list (readstring (t "name")) (readstring (t "password")) (N 'get 'address) )
                                (get-path))
                           (writestring (t "file") "")
                           (writestring (t "preview") "")

                           (set! currentpath (rest currentpath))
                           (writestring (t "directory") (first currentpath))
                           (update-dir)))))))
              (menuitem "delete file" buttoncolor (action (lambda(e)
                 (if (equal? "" (readstring (t "file")))
                     (warning "You must select a file before you can delete it")
                     (begin
                        (N 'send (readexpr (t "current netserver")) 'delete 
                                (list (readstring (t "name")) (readstring (t "password")) (N 'get 'address) )
                                (get-filename))
                        (writestring (t "file") "")
                        (writestring (t "preview") "")

                        (update-dir))))))
              (menuitem "update files/folders" buttoncolor (action (lambda(e)
                 (update-dir))))
              (menuitem "select netserver" (action (lambda(e) (.pack netserverwin) (.show netserverwin))))
           ))
       (border 
          (north 
           (col bgcolor 'center 'none
              (row 'center 'horizontal bgcolor (label "Network File Chooser" (HelveticaBold 32) bgcolor))))
          (south 
            (col bgcolor
                (border (north (label "PREVIEW")) 
                        (center (scrollpane (t "preview" (textarea 10 20)))))
                (splitrow 0 1 (label "directory") 
                    (t "directory" (textfield (string-append "/" user)  20 bgcolor (lambda(this) (.setEditable this #t)))))
                (splitrow 0 1 
                     (row bgcolor (label "file"))
                     (t "file" (textfield "" 40)))
                (row bgcolor 'center 'none
                   (t "OK" (button "OK" buttoncolor (action(lambda(e) 
                     (OKaction 
                           (get-filename)
                           (readstring (t "preview"))
                         )
                     (writestring (t "preview") "")
                     (.hide win)))))
                   (button "Cancel" buttoncolor (action(lambda(e) (.hide win)))))))
          (center 
           (t "sp0"
             (hsplit 200
              (t "sp1"
                (scrollpane
                   (t "dirlist" (jlist (lambda(x) 
                       (if (equal? x "..")
                           (if (not (null? (rest currentpath)))
                               (begin
                                 (set! currentpath (rest currentpath))
                                 (writestring (t "directory") (first currentpath))))
                           (begin
                             (appendstring (t "directory") (string-append "/" x))
                             (set! currentpath (cons (readstring (t "directory")) currentpath))
                           ))
                       (writestring (t "file") "")
                       (writestring (t "preview") "")
                       (update-dir))))))
               (t "sp2"
                (scrollpane
                   (t "filelist" (jlist (lambda(x) 
                          (N 'send (readexpr (t "current netserver")) 'open
                             (list (readstring (t "name")) (readstring (t "password")) (N 'get 'address))
                              (string-append (get-path) "/" x))
                          (writestring (t "file") x)))))))))
           white)))
  (define mkdirwin (dialog win #t "Make New Folder"
    (border 
      (north (row bgcolor 'center 'none (label "Make New Folder" (HelveticaBold 32) bgcolor)))
      (center
        (table 2 2 bgcolor
          (label "new folder name:") (t "new directory" (textfield "" 30 (action(lambda(e) (.hide mkdirwin)))))))
      (south
         (row 'none 'center bgcolor
            (button "OK" (action (lambda(e) (.hide mkdirwin))))
            (button "Cancel" (action (lambda(e) (writestring (t "new directory") "") (.hide mkdirwin)))))))))

  (define (filter F L)
    (if (null? L) ()
        (if (F (first L)) (cons (first L) (filter F (rest L))) (filter F (rest L)))))


  (.pack win)
  (.pack mkdirwin)

  ;; Now we add functionality to the netfilechooser


  (N 'add-listener 'error (lambda(cmd fs f)
     (display (list "**************error!!************" f))(newline)
     (warning f)))

   (N 'add-listener 'open-file (lambda(cmd fs . f)
    (display (list 'open-file cmd fs f)) (newline)
    (writestring (t "preview")
       (apply string-append (map (lambda(x) (string-append  x "\n")) (rest (first f)))))
;    (.paintImmediately (t "win") 0 0 (.getWidth win) (.getHeight win))
     ))

  (N 'add-listener 'open (lambda(cmd fs text)
     (writestring (t "preview") text)))

  (N 'add-listener 'netserver (lambda(cmd fs) 
     (set! current-netservers (cons (first h) current-netservers))
     (jlistwrite (t "netserverlist") current-netservers)
     (.validate (t "netserverlist"))
     (.validate (t "sp0"))
     (.validate (t "sp1"))
     (.validate (t "sp2"))
     (.paintImmediately (t "netserverlist") 0 0 (.getWidth (t "netserverlist")) (.getHeight (t "netserverlist")))
     ))
  (N 'add-listener 'files (lambda(cmd fs f) 
     (jlistwrite (t "dirlist")
         (cons ".."
         (map first (filter (lambda(x) (second x)) f))))
     (jlistwrite (t "filelist")
         (map first (filter (lambda(x) (not (second x))) f)))
     (.validate (t "dirlist"))
     (.validate (t "filelist"))
     (.validate (t "sp0"))
     (.validate (t "sp1"))
     (.validate (t "sp2"))
     (.paintImmediately (t "dirlist") 0 0 (.getWidth (t "dirlist")) (.getHeight (t "dirlist")))
     (.paintImmediately (t "filelist") 0 0 (.getWidth (t "filelist")) (.getHeight (t "filelist")))
     (display (list "recvd" f))(newline)
  ))
  (update-dir)


  (lambda R
    (case (first R)
      ((get) (case (second R)
               ((t) t)
               ((win) win)
               ((action) OKaction)
               (else #null)))
      ((set) (case (second R)
               ((action) (set! OKaction (third R)))
               (else #null)))
      ((save) 
              (N 'send 
                 (readexpr (t "current netserver"))
                 'save 
                 (list (readstring (t "name")) (readstring (t "password")) (N 'get 'address))
                 (string-append (readstring (t "directory")) "/" (readstring (t "file"))) 
                 (second R))
         )
     ))
)





