;; Fileserver.scm
;;  this is a simple fileserver service
;;
;;  To access the fileserver you must know its address FS-ADD
;;  which is of the form (list MACHINE PORT)
;;  The commands have the following form:
;;  (N 'send FS-ADD COMMAND '(USER PASSWORD RETURN-ADD) PATH A1 ... An)
;;  where the various commands and arguments are as follows:
;;   'dir    RETURNTO PATH
;;   'save   RETURNTO PATH DATA
;;   'open   RETURNTO PATH
;;   'rename RETURNTO PATH NEWPATH
;;   'delete RETURNTO PATH
;;   'mkdir  RETURNTO PATH
;;
;; current call from schemenet directory
;; java jscheme.REPL Networking.scm tatool/FileServer.scm '(make-fileserver port userfilehome authorizeall)'
;;
;; It provides a few types of authorization:
;;    authorizeall -- lets anyone do anything
;;    authorizecs2a -- does some minimal authorization for cs2a
;; I'll need to add a database authorization pretty soon, but for now it is OK.
;;

(import "jscheme.*")
(import "silk.*")

(define (dbg x) (display x) (newline))
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  main procedure  defaults:
;;  port 53642
;;  userfilehome = "."
;;  authorize="authorizeall"
;; (make-fileserver userfilehome authorize)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (start-local-fs) 
  (make-fileserver "localhost" 53642 "." (lambda R #t)))

(define (main args)
  (define (processargs arglist host port home authorize)
    (cond ((null? arglist) 
            (make-fileserver host port home authorize))
          ((equal? (first arglist) "-m")
           (processargs (rest(rest arglist)) (second arglist) port home authorize))
          ((equal? (first arglist) "-p")
           (processargs (rest(rest arglist)) host (java.lang.Integer.parseInt (second arglist)) home authorize))
          ((equal? (first arglist) "-h")
           (processargs (rest(rest arglist)) host port (second arglist) authorize))
          ((equal? (first arglist) "-a")
           (processargs (rest(rest arglist)) host port home
                      (eval (string->symbol (second arglist)))))
          (else (display (list "ERROR: unknown argument" (first arglist))
             (processargs  (rest arglist) port home authorize)))))
  (processargs (array->list args) "" 53642 "/home/t/tim/Research/Projects/grouptool/test" (usernameauth (cs2a-auth "!cs2a01!"))))
    

;; this gets the username from a string by removing the -XXXX suffix if there is one
(define (getusername S)
  (let ((z (.indexOf S "-")))
    (if (equal? z -1) S (.substring S 0 z))))

(define (cs2a-auth dbpw)
  ;; this gets the username from a string by removing the -XXXX suffix if there is one
  (define (getusername S)
    (let ((z (.indexOf S "-")))
      (if (equal? z -1) S (.substring S 0 z))))
  (lambda (user password)
  (let* (
   (dbhostport  "jdbc:mysql://odysseus.cs.brandeis.edu:3306/cs2a01")
   (dbuser  "cs2a01")
   (query
     (string-append "select password from pseudo where  unetid='" (getusername user) "';"))
   (result
    (run-query dbhostport dbuser dbpw query)))
  (equal? (first (second result)) (.toString password)))))


(define (usernameauth authorize) (lambda (N Room R)
             ;; R is a message of the form '(command returnto path arg1 arg2 ...)
             ;; where return to is '(username password (host port))
  (display (list 'usernameauth N Room R))(newline)
  (let* (
         (returnaddress (second R))
         (user (getusername (first returnaddress)))
         (password (second returnaddress))
         (OK (authorize user password))
        )
    (if (not OK)
        (begin (display (list "wrong password! " user password)) #f)
      (let* (
         (command (first R))
         (path (third R))
         (slashpos (.indexOf path "/" 1))
         (homedir (if (> slashpos 0) (.substring path 0 slashpos) "/"))
         (result 
           (or 
             (equal? homedir (string-append "/" user))
             (and (equal? command 'dir) (equal? homedir "/"))
           ))
        )
      (display (list 'usernameauthRESULT slashpos homedir user result "...." R)) (newline)
      result)))))

  

(define authorizeall  (lambda (N Room R)
  #t))

(define passwords-cs2a
  (let ((H (java.util.Hashtable.)))
    (.put H "/tjhickey" "x+y+z")
    (.put H "/guest" "")
    (.put H "." "")
    (.put H "/" "")
    (.put H "" "")
     H))

;;  R has the form (COMMAND (USER PASSWD RETURN-ADD) PATH A1 ... An)
(define authorizecs2a (lambda (N Room R)
  (let* ((path (third R))
         (slashpos (.indexOf path "/" 1))
         (homedir (if (> slashpos 0) (.substring path 0 slashpos) "/")))
    (let ((pw (.get passwords-cs2a homedir)))
        (or (equal? pw "") (equal? pw (second (second R)))))
)))
 
(define (toStringQ S)  (jscheme.REPL.printToString S #t))

(define (make-fileserver host port userfilehome authorize)
  (define N (make-servent port))

  (define filesep (java.lang.System.getProperty "file.separator"))

  (define add-authorized-listener (lambda (N Room F G)
    (N 'add-listener Room 
     (lambda R     ;; R is a message of the form (cmd path password ....)
       (if (apply authorize (list  N  Room R))
           (apply F R)
           (begin (apply G R) (display (list 'security-error (cons Room R)))(newline)))
    ))
  ))


  (define notifyPasswordError
   (lambda (cmd returnto . Rest)
       (display (list "sending error message"))(newline)
       (display (list N 'send (get-add returnto) 'error (N 'get 'address) "password incorrect"))(newline)
       (N 'send (get-add returnto) 'error (N 'get 'address) "you are not authorized to view these files")))

  (define (get-add L) (third L))

 (if (not (equal? host "")) (N 'set 'host host))

 (add-authorized-listener N 'dir 
  (lambda (cmd returnto path)
  (dbg (list 'indir cmd returnto path))
  (tryCatch
   (begin
    (if (or (equal? path "") (equal? path "/"))
      (let ((F (java.io.File. userfilehome)))
        (dbg (list "file check" F (.exists F)))
        (if (not (.exists F)) (.mkdir F))))
    (let*  (
           (filename (string-append userfilehome path))
           (path (java.io.File. filename))
           (files (.list path))
           (dir-info 
             (map 
                (lambda(x) 
                    (list x (.isDirectory (java.io.File. (string-append filename  filesep x)))))
                (array->list files))))
      (display (list 'dir filename "-->" dir-info))(newline)
      (display (list 'dir 'files "-->" files))(newline)
      (display (list 'dir 'path "-->" path))(newline)
      (display (list 'N 'send (get-add returnto) 'files (N 'get 'address) dir-info))(newline)
      (N 'send (get-add returnto) 'files  (N 'get 'address) dir-info)))
    (lambda(e) 
       (display (list "ERROR in dir" (toStringQ e))) (newline))))

    notifyPasswordError
   )


  (add-authorized-listener N 'save 
   (lambda (cmd returnto savepath data)
   (tryCatch
    (let* (
           (file (java.io.File. (string-append userfilehome savepath)))
           (outfile (java.io.PrintWriter. (java.io.FileWriter. file)))
           (len (.length data))
          )
       (let loop ((pos 0))
           (let (( nextpos (.indexOf data "\n" pos)))
               (if (= nextpos -1)
                   (.print outfile (.substring data pos))
                   (begin
                      (.println outfile (.substring data pos nextpos))
                      (loop (+ 1 nextpos))))))
       (.flush outfile)
       (.close outfile))
    (lambda(e) 
       (N 'send (get-add returnto) 'error  (N 'get 'address) (list "ERROR id dir" (toStringQ e)))
       (display (list "ERROR in dir" (toStringQ e))) (newline))))

    notifyPasswordError
    
)


  (add-authorized-listener N 'open 
    (lambda (cmd returnto path)
    (tryCatch
        (let*  (
               (filename (string-append userfilehome path))
               (infile (java.io.BufferedReader. (java.io.FileReader. filename)))
               (data
                  (let loop ((x (.readLine infile)))
                      (if (equal? #null x)
                          ()
                          (cons x (loop (.readLine infile))))))
             )
         (N 'send (get-add returnto)  'open-file  (N 'get 'address) (cons path data)))
     (lambda(e) 
         (N 'send (get-add returnto)  'open-file  (N 'get 'address) (list path e)))))

    notifyPasswordError
    )



  (add-authorized-listener N 'rename 
    (lambda (cmd returnto path newpath)
    (tryCatch
        (let*  (
                 (srcfile (java.io.File. (string-append userfilehome path)))
                 (dstfile (java.io.File. (string-append userfilehome newpath)))
               )
           (.renameTo srcfile dstfile))
     (lambda(e) 
         (N 'send (get-add returnto)  'error  (N 'get 'address) (list 'rename path e)))))

    notifyPasswordError
    )



  (add-authorized-listener N 'delete 
    (lambda (cmd returnto path)
    (tryCatch
        (let*  (
                 (file (java.io.File. (string-append userfilehome path)))
               )
           (.delete file))
     (lambda(e) 
         (N 'send (get-add returnto)  'error  (N 'get 'address) (list 'delete path e)))))

    notifyPasswordError
    )


  (add-authorized-listener N 'mkdir 
    (lambda (cmd returnto path)
    (tryCatch
        (let*  (
                 (file (java.io.File. (string-append userfilehome path)))
               )
           (.mkdir file))
     (lambda(e) 
         (N 'send (get-add returnto)  'open-file  (N 'get 'address) (list path e)))))

    notifyPasswordError
    )

  (N 'add-listener 'quit
     (lambda (cmd returnto)
       (if (equal? (first returnto) 'cs2aroot) (java.lang.System.exit 0))))

  (N 'add-super-listener (lambda R (display (list 'super-listener R)) (newline)))
 N
)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; rundemo ---
;; This gives an example of how to create a file server
;; and how to use it. Run this and look at the return values
;; to get an idea of which messages are sent back to the client
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (rundemo)
  (define the-fs (tryCatch (main #()) (lambda(e) 'already-up)))
  (define N (make-servent 0))
  (define FS '("localhost" 53642))
  (define ME `('me 'ok ("localhost" ,(N 'get 'port))))
  (N 'add-super-listener (lambda R (display (list 'super-listener R))(newline)))
  (N 'send FS 'dir ME "/")
  (N 'send FS 'mkdir ME "/jnkk")
  (N 'send FS 'dir ME "/")
  (N 'send FS 'save ME "/jnkk/abc" "This is a\ntest\nSee??\n")
  (N 'send FS 'dir ME "/jnkk")
  (N 'send FS 'open ME "/jnkk/abc")
  (N 'send FS 'rename ME "/jnkk/abc" "/jnkk/cba")
  (N 'send FS 'dir ME "/jnkk")
  (N 'send FS 'delete ME "/jnkk/cba")
  (N 'send FS 'dir ME "/jnkk")
  (N 'send FS 'delete ME "/jnkk")
  (N 'send FS 'dir ME "/")
)


(define N (make-servent 0))
(N 'add-super-listener (lambda R (display (list 'runtest R)) (newline)))

(define (runtest user pass path)
  (N 'send (list "localhost" 53642) 'dir (list user pass (N 'get 'address)) path))

