(jlib.Swing.load)
(load "tat/Networking.scm")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Registrar-client
;;
;; This program creates a registration window/buddy list
;; It allows the user to select which registrar they want
;; to use. Once they select a registrar they get a jlist
;; of all the users on the registrar. When they click
;; on a user the BuddyAction is called on that user
;;
;;   N is a servent connection
;;
;;  (BuddyAction x) is what will be called if the
;;    user clicks on buddy x, where x=(name (host port))
;;
;;  This abstracts away the problem of connecting with other
;;  users.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

 (define (make-regwin N BuddyAction)

  (define t (maketagger))
  (define name "guest")
  (define FS "")
  (define REG "")
  ;; this is the way in which the tool determines its own IP address and port
  ;; some JVM's are not able to determine their own IP address 
  (define (verify-address N REG)
   (display (list "verifying: " N REG))(newline)
   (tryCatch
    (let ((x (send-line (first REG) (second REG) "ping")))
      (N 'set 'host x)
      (N 'get 'address)
      (display (list "verifying: " (N 'get 'address))) (newline)
     )
    (lambda(e) (display (list "verification error:" e))(newline) "localhost")))

  (define (register Name Password)
    (N 'send REG
       'login Name Password (N 'get 'address)))


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;  ... regwin ...
  ;; this is the registration window.
  ;; It prompts the user for their name/password/group
  ;; and then it uses that information to send a 'login message to
  ;; the registrar
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

   (define classdata '( ("localhost" 53642) ("localhost" 23232)))

 
   (define groupwin (window "select group"
        (table 5 2
          (label "registrar host") (t "reghost" (textfield "localhost" 20))
          (label "registrar port") (t "regport" (textfield "23232" 20))
          (label "fileserver host") (t "fshost" (textfield "localhost" 20))
          (label "fileserver port") (t "fsport" (textfield "53642" 20))
          (button "OK" (action (lambda(e) 
            (set! classdata (list (list (readstring (t "fshost")) (readexpr (t "fsport")))
                                  (list (readstring (t "reghost")) (readexpr (t "regport")))))
            (.hide groupwin)))))))


  (define regwin
   (window "Registration" white
    (lambda(this)
     (.addWindowListener this (Listener. (lambda(e)
         (if (equal? (.getID e) java.awt.event.WindowEvent.WINDOW_CLOSING$)
             (java.lang.System.exit 0))))))
   (menubar (menu "Options" (menuitem "select registrar" (action (lambda(e) (.pack groupwin) (.show groupwin))))))
   (border
     (north (col 'center 'none (label "TA-chat tool")))
     (center
       (table 3 2
         (label "username") (t "username" (textfield name 20))
         (label "password") (t "password" (javax.swing.JPasswordField. "thanks" 20))

         (label "group")    

         ;;; abstract this expression, move data to a parameter.......
         (t "group" (choice "offline" "cs2a" "typ" "tatdev" "guest"
           (action (lambda(e) 
            (set! classdata
              (case (readstring (t "group"))
                (("offline") '( ("localhost" 53642) ("localhost" 23232)))
                (("cs2a") '( ("tat.cs.brandeis.edu" 53642) ("tat.cs.brandeis.edu" 23232)))
                (("typ") '( ("frege.cs-i.brandeis.edu" 53642) ("frege.cs-i.brandeis.edu" 23232)))
                (("tatdev") '( ("frege.cs-i.brandeis.edu" 53643) ("frege.cs-i.brandeis.edu" 23233)))
                (("guest")  '( ("russell.cs-i.brandeis.edu" 53642) ("russell.cs-i.brandeis.edu" 23232)))
             ))))))))
     (south 
       (button "register" (action (lambda(e)
         (.hide regwin)
         (set! name (readstring (t "username")))
         ;; first we set the registrar and file server for the class
         (let* (  
            (group (readstring (t "group")))
               )
           (set! FS (first classdata))
           (set! REG (second classdata)))
         ;; next we verify the address of the servent using the REGISTRAR
         ;; Some JVM's can't determine their own IP address, but a remote machine can
         (verify-address N REG)
         ;; now we register with the registrar and show the main window for starting chats,tats,etc.
         (register (readstring (t "username")) (readstring (t "password")))
         (.setTitle mainchatwin 
             (string-append "Main Chat Window for " 
                  (readstring (t "username"))))
         (.pack mainchatwin) (.show mainchatwin))))))))


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;  ... mainchatwin ...
  ;; this window maintains the buddy list
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define mainchatwin (window "main chat win" white
   (lambda (this)     (.addWindowListener this (Listener. (lambda(e)
         (if (equal? (.getID e) java.awt.event.WindowEvent.WINDOW_CLOSING$)
             (java.lang.System.exit 0))))))
    (menubar 
      (menu "File"
         (menuitem "quit"
           (action (lambda(e)
              (N 'send REG 'logout 
                  (readstring (t "username")) 
                  (readstring (t "password")))
              (.hide mainchatwin)
              (java.lang.System.exit 0) )))))


    (border
      (north (col 
               (row 'none 'center (label "Buddylist (version date: 1/30/02)" (HelveticaBold 18) white))
         ))
      (center (t "au" (scrollpane (t "active-users" (jlist (lambda(x) (BuddyAction x))))))))))

(N 'add-listener 'are-you-there (lambda R
  (N 'send REG 'i-am-here name)))
(N 'add-listener 'add-user (lambda R 
    (jlistwrite (t "active-users") 
        (map (lambda(x) (list (first x) (second x))) (second R)))
    (.validate (t "au"))  ))
(N 'add-listener 'drop-user (lambda R 
    (jlistwrite (t "active-users") 
        (map (lambda(x) (list (first x) (second x))) (second R)))
    (.validate (t "au"))  ))


 (.pack regwin)
 (.show regwin)
 regwin
)

(define (demoaction N x)
  (define w (window "test" (label x)))
  (.resize w 300 300)
  (.show w)
)



(define (go2 M)
 (define N (make-servent 0))
 (define (mybuddyaction x)
  (define t (maketagger))
  (define GS (make-group-server 0))     ;; here we create a group server on any available port
  (define GS-port (GS 'get 'port))      ;; and we find out which port was used
  (define N (make-servent 0))           ;; this is the main network interface for the user


  (define FS ()) ;(list "tat.cs.brandeis.edu" 53642))
  (define REG ()) ;(list "silk.cs.brandeis.edu" 23232))  ;; here we redefine the registrar



  (define actionwin (window "action window"
    (border
      (south
        (col
          (t "selection" (textfield (.toString x) 0))
          (row 
            (button "editor" (action(lambda(e) (make-editor))))

            (button "ask for help" (action (lambda(e)
                (let ((user/address (readexpr (t "selection"))))
                  (N 'send (second user/address) 'want-to-help-me
                        GS-port
                        (list (readstring (t "username")) 
                           (N 'get 'address))
                        'ta-mode
                     )))))
                    
            (button "work together" (action (lambda(e)
                (let ((user/address (readexpr (t "selection"))))
                  (N 'send (second user/address) 'want-to-help-me
                        GS-port
                        (list (readstring (t "username")) 
                           (N 'get 'address))
                        'work-together
                     )))))
                    
            (button "chat" (action (lambda(e) 
                (let ((user/address (readexpr (t "selection"))))
                  (N 'send (second user/address) 'want-to-chat
                        GS-port
                        (list (readstring (t "username")) 
                           (N 'get 'address))))))))
           (row 'none 'west (label "chat policy" white) (t "autochat" 
               (choice "accept all chats" "ask me" "refuse all chats" white)))
           (row 'none 'west (label "tat policy" white) (t "autotat" 
               (choice "accept all tats" "ask me" "refuse all tats" white)))
           (row (label "refusal message") (t "refusal message" (textfield "I'm away from my computer." 50)))
        ))

      )))

     (.pack actionwin)
     (.show actionwin)
   )

 (define w (make-regwin N mybuddyaction))
 w
)

 


(define (go M)
  (define N (make-servent 0))
  (define w (make-regwin N (lambda R (display (list M R))(newline))))
  w)


