;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GroupExp.scm
;;
;;  This is the main window for the grouptool experimental platform
;;  The student and ta windows need to be refactored to eliminate
;;  the unneeded duplication of code.
;;
;;  Most of the code here is for creating and managing the 
;;  registration windows, and the main TA and student windows
;;  This file contains the GUI code for
;;    -- the login window
;;    -- the registrar window for picking users to work with
;;    -- the ta, student, and chat windows
;;  It also contains networking code for managing
;;    -- protocols for tas/students to agree to
;;       chat/tat/work together
;;  This networking is fairly complex and it would be nice to
;;  isolate it somehow from the GUI code.
;;
;;  The code for the shared components (shared textareas and whiteboards)
;;  is in separate files.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(jlib.Swing.load)
(define jlib.JLIB.load (lambda() #t)) ;; this prevents students crashing system
                                      ;; by loading the AWT windowing package
                                      ;; thereby replacing the Swing package  

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; Set Class Data
  ;; Stores basic data about class in form:
  ;;    (list FileServerHost/Port ClassRegistrar)
  ;; It gets the class from the (t "group") component in the
  ;; initial registration window.
  ;; This should probably be in some other file, like CS2aTatClient.scm
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define (classdata)
    (case (readstring (t "group"))
     (("offline") '( ("localhost" 53642) ("localhost" 23232)))
     (("cs2a") '( ("tat.cs.brandeis.edu" 53642) ("tat.cs.brandeis.edu" 23232)))
     (("typCosi") '( ("frege.cs-i.brandeis.edu" 53642) ("tat.cs.brandeis.edu" 23232)))
     (else '( ("tat.cs.brandeis.edu" 53642) ("tat.cs.brandeis.edu" 23232)))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  (start-tatclient NAME)
;; This is how one starts up the group experiment tool
;; You give it a default user NAME and default address of the registrar
;;
;; The user can then change
;; Note: the tatclient creates one group-server and one servent at the start,
;; For each new student/ta connection, it creates a new servent and a new
;; group client, but no new group servers. 
;;
;; The main tatclient window is a tabbed pane with one tab for
;; each live network connection.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (start-tatclient name)

  (define FS (list "tat.cs.brandeis.edu" 53642))
  (define REG (list "silk.cs.brandeis.edu" 23232))  ;; here we redefine the registrar

  (define t (maketagger))
  (define GS (make-group-server 0))     ;; here we create a group server on any available port
  (define GS-port (GS 'get 'port))      ;; and we find out which port was used

  (define remotecolor (color 230 230 255)) ;; this is used to signify an textarea you can view but not modify

  
  (define N (make-servent 0))           ;; this is the main network interface for the user

  ;; this is the way in which the tool determines its own IP address and port
  ;; some JVM's are not able to determine their own IP address 
  (define (verify-address N REG)
   (display (list "verifying: " N REG))(newline)
   (tryCatch
    (let ((x (send-line (first REG) (second REG) "ping")))
      (N 'set 'host x)
      (N 'get 'address)
      (display (list "verifying: " (N 'get 'address))) (newline)
     )
    (lambda(e) (display (list "verification error:" e))(newline) "localhost")))

  (define (register Name Password)
    (N 'send REG
       'login Name Password (N 'get 'address)))


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;  ... regwin ...
  ;; this is the REGISTRATION WINDOW that gets the
  ;; students name/password/class
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define regwin  (window "Registration" white
   (lambda(this)
     (.addWindowListener this (Listener. (lambda(e)
         (if (equal? (.getID e) java.awt.event.WindowEvent.WINDOW_CLOSING$)
             (java.lang.System.exit 0))))))
   (border
     (north (col 'center 'none (label "TA-chat tool")))
     (center
       (table 3 2
         (label "username") (t "username" (textfield name 20))
         (label "password") (t "password" (javax.swing.JPasswordField. "thanks" 20))
         (label "group")    (t "group" (choice "offline" "cs2a" "typ" "tatdev" "guest"))))
     (south 
       (button "register" (action (lambda(e)
         (.hide regwin)
         (set! name (readstring (t "username")))
         ;; first we set the registrar and file server for the class
         (let* (  
            (group (readstring (t "group")))
            (classdata 
              (case group
                (("offline") '( ("localhost" 53642) ("localhost" 23232)))
                (("cs2a") '( ("tat.cs.brandeis.edu" 53642) ("tat.cs.brandeis.edu" 23232)))
                (("typ") '( ("frege.cs-i.brandeis.edu" 53642) ("frege.cs-i.brandeis.edu" 23232)))
                (("tatdev") '( ("frege.cs-i.brandeis.edu" 53643) ("frege.cs-i.brandeis.edu" 23233)))
                (("guest")  '( ("russell.cs-i.brandeis.edu" 53642) ("russell.cs-i.brandeis.edu" 23232))))))
           (set! FS (first classdata))
           (set! REG (second classdata)))
         ;; next we verify the address of the servent using the REGISTRAR
         ;; Some JVM's can't determine their own IP address, but a remote machine can
         (verify-address N REG)
         ;; now we register with the registrar and show the main window for starting chats,tats,etc.
         (register (readstring (t "username")) (readstring (t "password")))
         (.setTitle mainchatwin 
             (string-append "Main Chat Window for " 
                  (readstring (t "username"))))
         (.pack mainchatwin) (.show mainchatwin))))))))


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;  ... mainchatwin ...
  ;; this window maintains the BUDDYLIST and
  ;; allows the student to edit files, chat or ask for help
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  (define mainchatwin (window "main chat win" white
   (lambda (this)     (.addWindowListener this (Listener. (lambda(e)
         (if (equal? (.getID e) java.awt.event.WindowEvent.WINDOW_CLOSING$)
             (java.lang.System.exit 0))))))
    (menubar 
      (menu "File"
         (menuitem "quit"
           (action (lambda(e)
              (N 'send REG 'logout 
                  (readstring (t "username")) 
                  (readstring (t "password")))
              (.hide mainchatwin)
              (java.lang.System.exit 0) )))))


    (border
      (north (col 
               (row 'none 'center (label "TaTool Buddylist (version date: 11/13/01)" (HelveticaBold 18) white))
         ))
      (center (t "au" (scrollpane (t "active-users" (jlist (lambda(x)
                 (writestring (t "selection") x)))))))
      (south
        (col
          (t "selection" (textfield "" 0 (lambda(this) (.setVisible this #f)) ))
          (row 
            (button "editor" (action(lambda(e) (make-editor))))

            (button "ask for help" (action (lambda(e)
                (let ((user/address (readexpr (t "selection"))))
                  (N 'send (second user/address) 'want-to-help-me
                        GS-port
                        (list (readstring (t "username")) 
                           (N 'get 'address))
                        'ta-mode
                     )))))
                    
            (button "work together" (action (lambda(e)
                (let ((user/address (readexpr (t "selection"))))
                  (N 'send (second user/address) 'want-to-help-me
                        GS-port
                        (list (readstring (t "username")) 
                           (N 'get 'address))
                        'work-together
                     )))))
                    
            (button "chat" (action (lambda(e) 
                (let ((user/address (readexpr (t "selection"))))
                  (N 'send (second user/address) 'want-to-chat
                        GS-port
                        (list (readstring (t "username")) 
                           (N 'get 'address))))))))
           (row 'none 'west (label "chat policy" white) (t "autochat" 
               (choice "accept all chats" "ask me" "refuse all chats" white)))
           (row 'none 'west (label "tat policy" white) (t "autotat" 
               (choice "accept all tats" "ask me" "refuse all tats" white)))
           (row (label "refusal message") (t "refusal message" (textfield "I'm away from my computer." 50)))
        ))

   )))
 


  (define mytatabbedpane (javax.swing.JTabbedPane.))
  (define mytawin (window "ta window" mytatabbedpane))


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; ... make-super-ta ...
  ;; this creates a ta window connected to a given student
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (make-super-ta student-name host servent-port groupserver-port cooperation-mode) ;Name Net)
  (define Net (make-group-client "ta" student-name host groupserver-port))
  (define tt (maketagger))

 (define mytapanel
  (border
   (north 
     (col
       (label (string-append "ta: " name " helping " student-name " at "
                             host "/" servent-port) (HelveticaBold 18) white)
       ))
     (south
        (col 'none 'north white
         (row 'none 'west white
           (button "show chats" (action (lambda(e) (.show (tt "chats"))
                        (.setDividerLocation (tt "h0") 0.8)
                        (.validate mytapanel))))
           (button "hide chats" (action (lambda(e) (.hide (tt "chats"))
                        (.setDividerLocation (tt "h0") 1.0)
                        (.validate mytapanel))))
           (button "show tats" (action (lambda(e) (.show (tt "first")) (.show (tt "third"))
                        (.setDividerLocation (tt "h1") 0.2)
                        (.setDividerLocation (tt "h2") 0.75)
                        (.validate mytapanel)
                      )))
           (button "hide tats" (action (lambda(e) (.hide (tt "first")) (.hide (tt "third")) 
                        (.setDividerLocation (tt "h1") 0.0)
                        (.setDividerLocation (tt "h2") 1.0)
                        (.validate mytapanel))))
           )))

   (center
    (tt "h0"
    (vsplit 200    
     (tt "h1"
      (hsplit 75
        (border (north (label "student comments")) (center
        (tt "first" (scrollpane (tt "stacomm" (remote-textarea Net 'stcomm 10 20 remotecolor))))))
       (tt "h2" (hsplit 250
        (border
          (north (label "student program")) 
          (south  (schemebuttonpanel tt))
          (center
            (tt "second" (scrollpane (tt "staprog" 
                          (if (equal? cooperation-mode 'work-together)
                              (both-textarea Net 'stprog 'blue 10 40)
                              (remote-textarea Net 'stprog 10 20 remotecolor))
                       )))))
        (border (north  (label "ta comments")) (center
          (tt "third" (scrollpane (tt "rtacomm" (shared-textarea Net 'tacomm 10 20) )))))))))
    (tt "chats"
     (hsplit 150
        (label "") ; (wbpanel "ta" "test" host groupserver-port)        
        (p2p-chat-panel (list student-name (list host servent-port))))))))))


    
    ;; this listener adds lines to the ta comment window whenever lines
    ;; are typed into the student window
    (define insert-listener (lambda (cmd offset str)
       (define (number-lines str)
         (define (loop2 N pos)
           (let ((q (.indexOf str "\n" pos)))
            (if (= q -1)
               N
               (loop2 (+ N 1) (+ q 1)))))
         (loop2 0 0))
       (define (make-blank-lines N) 
         (define (loop3 N L)
           (if (<= N 0) L 
               (loop3 (- N 1) (cons "\n" L))))
         (apply string-append (loop3 N ())))
        (display
                   (.getLineOfOffset (tt "staprog") offset)) (newline)
        (display 
                   (.getLineStartOffset (tt "staprog") (.getLineOfOffset (tt "staprog") offset)))
        (newline)
        (.insert (tt "rtacomm") 
               (make-blank-lines (number-lines str))
               (.getLineStartOffset (tt "rtacomm") (.getLineOfOffset (tt "staprog") offset))
          )))


    ;; this code makes the three textareas scroll together!

    (define (listener leader follower1 follower2)
	(Listener. (lambda(e)
		     (.setValue follower1 (.getValue leader))
		     (.setValue follower2  (.getValue leader))
                   )))


     (display (list "adding the scrolllisteners"))(newline)

    (tt "first-scroll" (.getVerticalScrollBar (tt "first")))
    (tt "second-scroll"(.getVerticalScrollBar (tt "second")))
    (tt "third-scroll" (.getVerticalScrollBar (tt "third")))

     (display (list "adding the adjust listeners"))(newline)

    (.addAdjustmentListener (tt "second-scroll") (listener (tt "second-scroll") (tt "second-scroll") (tt "third-scroll")))
    (.addAdjustmentListener (tt "first-scroll") (listener (tt "first-scroll") (tt "first-scroll") (tt "third-scroll")))
    (.addAdjustmentListener (tt "third-scroll") (listener (tt "third-scroll") (tt "second-scroll") (tt "first-scroll")))



  (display (list "adding the listener"))(newline)
  (Net 'add-listener (list 'stprog 'insert 'shared-textarea) insert-listener)
  (display (list "added the listener"))(newline)

    ;; here we insert a few black lines into the ta comment window
    (.insert (tt "rtacomm") "\n\n\n" 0)



   ;; uncomment the next line to listen to all the traffic on the servent for this client.
;  (Net 'add-super-listener (lambda R (display (list name "heard" R))(newline)))

   (.add mytatabbedpane (string-append "st:" student-name) mytapanel)
   (.pack mytawin)
   (.show mytawin)

    ;; this is a hack to force the window into a nice size on my Linux boxes....
    (.setDividerLocation (tt "h1") 0.2)
    (.setDividerLocation (tt "h2") 0.75)
    (.validate mywin)
    (.setDividerLocation (tt "h1") 0.2)
    (.setDividerLocation (tt "h2") 0.75)
    (.validate mywin)

    ;; here we hide the "chats" by default
    (.setDividerLocation (tt "h0") 1.0)
    (.validate mywin)
   mytapanel
)



  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; ... make-super-student ...
  ;; this creates a student window connected to a given ta
  ;; This needs to be refactored into the make-super-ta
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


  (define  (make-super-student name ta-name host servent-port cooperation-mode)
    (define jnk (display (list 'make-super-student name host servent-port)) (newline))
    (define zz (make-group-client "student" name "localhost" GS-port))
    (define Net zz)
    
    (define tt (maketagger))

    (define N 
      (let ((x (make-servent 0)))
         (verify-address x REG)
         x))
;         (display (list "sending to REG" REG) ) (newline)
;          (x 'set 'host (send-line (first REG) (second REG) "ping"))
;         (display (list "sent to REG" REG) ) (newline)
;          x))

    (define netfileserver 
        (netfilechooser 
           (readstring (t "username")) (readstring (t "password"))
            mainchatwin N FS (lambda R (display R)(newline))))


    (define schemeanswin 
      (let ((w (window "scheme answers" 
                  (scrollpane (tt "scheme-ans" (textarea 20 40 white))))))
         (.pack w)
         w))

    (define mywin 
       (window (string-append "st: " name 
                     " getting help from " ta-name ":" 
                       host "/" servent-port) white
      (menubar
       (tt "student-file-menu"(menu "File"))

       (fontmenu (lambda() (list (tt "staprog") (tt "stacomm") (tt "rtacomm"))))

       (menu "Server"
         (menuitem "download" (action (lambda(e) 
               (let ((t (netfileserver 'get 't)))
                  (writestring (t "OK") "download file")
                  (netfileserver 'set 'action (lambda (filename text)
                    (writestring (tt "staprog") text)))
                  (.show (netfileserver 'get 'win))))))
          (menuitem "upload" (action (lambda(e) 
               (let ((t (netfileserver 'get 't)))
                  (writestring (t "OK") "upload file")
                  (netfileserver 'set 'action (lambda R
                        (netfileserver 'save (readstring (tt "staprog")))))
                  (.show (netfileserver 'get 'win))))))))


   (border
     (north 
       (col
         (label (string-append name 
                     " getting help from " ta-name ":" 
                       host "/" servent-port) white)
;         (schemebuttonpanel tt)
          ))
     (south
        (col 'none 'north white
         (row 'none 'west white
           (button "show chats" (action (lambda(e) (.show (tt "chats"))
                        (.setDividerLocation (tt "h0") 0.8)
                        (.validate mywin))))
           (button "hide chats" (action (lambda(e) (.hide (tt "chats"))
                        (.setDividerLocation (tt "h0") 1.0)
                        (.validate mywin))))
           (button "show tats" (action (lambda(e) (.show (tt "first")) (.show (tt "third"))
                        (.setDividerLocation (tt "h1") 0.2)
                        (.setDividerLocation (tt "h2") 0.75)
                        (.validate mywin)
                      )))
           (button "hide tats" (action (lambda(e) (.hide (tt "first")) (.hide (tt "third")) 
                        (.setDividerLocation (tt "h1") 0.0)
                        (.setDividerLocation (tt "h2") 1.0)
                        (.validate mywin))))
           )))
    (center
     (tt "h0"
      (vsplit 200
	  (tt "h1" (hsplit 75
                     (border (north (label "student comments")) (center
                        (tt "first" (scrollpane (tt "stacomm" (shared-textarea Net 'stcomm 10 20 ))))))
		   (tt "h2" (hsplit 250
                          (border (north (label "student program")) 
                            (south (schemebuttonpanel tt))
                            (center
			     (tt "second" (scrollpane (tt "staprog" 
                                           (if (equal? cooperation-mode 'work-together)
                                             (both-textarea Net 'stprog 'green 10 40)
                                             (shared-textarea Net 'stprog 10 20))
                                )))))
                          (border (north (label "ta comments")) (center
			     (tt "third" (scrollpane (tt "rtacomm" (remote-textarea Net 'tacomm 10 20 remotecolor))))))))))
       (tt "chats" 
        (hsplit 150
         (label "") ; (wbpanel "student" "test" "localhost" GS-port) 
	 (p2p-chat-panel (list ta-name (list host servent-port)))))))))))


    (define (listener leader follower1 follower2)
	(Listener. (lambda(e)
		     (.setValue follower1 (.getValue leader))
		     (.setValue follower2  (.getValue leader))
)))

    (addFileMenuItems (tt "student-file-menu") mywin (tt "staprog"))




    (tt "first-scroll" (.getVerticalScrollBar (tt "first")))
    (tt "second-scroll"(.getVerticalScrollBar (tt "second")))
    (tt "third-scroll" (.getVerticalScrollBar (tt "third")))

    (.addAdjustmentListener (tt "second-scroll") (listener (tt "second-scroll") (tt "second-scroll") (tt "third-scroll")))
    (.addAdjustmentListener (tt "first-scroll") (listener (tt "first-scroll") (tt "first-scroll") (tt "third-scroll")))
    (.addAdjustmentListener (tt "third-scroll") (listener (tt "third-scroll") (tt "second-scroll") (tt "first-scroll")))


    (display (list "Net address: " (N 'get 'address))) (newline)
;    (Net 'add-super-listener (lambda R (display (list name "heard" R))(newline)))
;    (N 'add-super-listener (lambda R (display (list name "heard" R))(newline)))
;    (.resize mywin 800 400)
    (.pack mywin)
    (.show mywin)

    ;; this is a hack to force the window into a nice size on my Linux boxes....
    (.setDividerLocation (tt "h1") 0.2)
    (.setDividerLocation (tt "h2") 0.75)
    (.validate mywin)
    (.setDividerLocation (tt "h1") 0.2)
    (.setDividerLocation (tt "h2") 0.75)
    (.validate mywin)

    ;; here we hide the "chats" by default
    (.setDividerLocation (tt "h0") 1.0)
    (.validate mywin)
    mywin
)

     

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; ... schemebuttonmenu  ...
  ;; this creates the scheme IDE buttons relative to a given textarea
  ;;   (tt "staprog")
  ;; ...this could go in another file as we may want ot have
  ;; different kinds of functionality, not just scheme...
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


  (define (schemebuttonpanel tt)
   (let* (
      (greenhighlights (javax.swing.text.DefaultHighlighter$DefaultHighlightPainter. green))
      (yellowhighlights (javax.swing.text.DefaultHighlighter$DefaultHighlightPainter. yellow))
      (schemeanswin 
        (let ((w (window "scheme answers" 
                    (scrollpane (tt "scheme-ans" (textarea 20 40 white))))))
           (.pack w)
           w))
         )
      (row white (grid 1 3 white
          (button "highlight"
             (action (tt "action_highlight" (lambda(e)
               (let* ((ProgramTA (tt "staprog"))
                      (T (.getText ProgramTA))
                      (P (.getCaretPosition ProgramTA))
                      (Bs (current-expr-bounds P (.getText ProgramTA)))
                      (endpos (second Bs))
                      (startpos (first Bs))
                      (program_highlighter (.getHighlighter (tt "staprog"))))
                   (.removeAllHighlights program_highlighter)
                   (cond 
                     ((and (< startpos P) (< P endpos))
                      (.addHighlight program_highlighter startpos P yellowhighlights)
                      (.addHighlight program_highlighter P  endpos greenhighlights))
                     ((< startpos endpos)
                      (.addHighlight program_highlighter startpos endpos yellowhighlights)))
                )))))

          (button "eval" 
                   (action (lambda(e) 
                      (.show schemeanswin)
                      (for-each (lambda(x) (appendlnexpr (tt "scheme-ans") 
                                    (string-append ">>> " x "\n --> " 
                                         (tryCatch (jlibeval x) 
                                                   (lambda(e) (list "***ERROR***" e))))))
                         (string->exprlist (readstring (tt "staprog")))))))


          (button "unhighlight" (action (lambda(e)
            (let (
                (program_highlighter (.getHighlighter (tt "staprog")))
                 )
              (.removeAllHighlights program_highlighter)))))

      ))))






  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; ... make-editor  ...
  ;; this creates an editor window
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


  (define  (make-editor)
    (define tt (maketagger))

    (define N 
      (let ((x (make-servent 0)))
         (verify-address x REG)
         x))
;         (display (list "sending to REG" REG) ) (newline)
;          (x 'set 'host 
;           (tryCatch
;             (send-line (first REG) (second REG) "ping")
;             (lambda(e) "localhost")))
;         (display (list "sent to REG" REG) ) (newline)
;          x))


    (define netfileserver 
        (netfilechooser 
           (readstring (t "username")) (readstring (t "password"))
            mainchatwin N FS (lambda R (display R)(newline))))



    (define mywin (let ((win
       (window "editor window"
        (menubar
        (tt "editor-file-menu" (menu "File"))
        (fontmenu (lambda() (list (tt "staprog"))))
        (menu "Server"
          (menuitem "download" (action (lambda(e) 
               (let ((t (netfileserver 'get 't)))
                  (writestring (t "OK") "download file")
                  (netfileserver 'set 'action (lambda (filename text)
                    (writestring (tt "staprog") text)))
                  (.show (netfileserver 'get 'win))))))
          (menuitem "upload" (action (lambda(e) 
               (let ((t (netfileserver 'get 't)))
                  (writestring (t "OK") "upload file")
                  (netfileserver 'set 'action (lambda R
                        (netfileserver 'save (readstring (tt "staprog")))))
                  (.show (netfileserver 'get 'win))))))))
      (border
         (north 
           (col
             (label "Scheme Editor/Evaluator" (HelveticaBold 32) white)
             (schemebuttonpanel tt)
            ))
         (center 
             (scrollpane (tt "staprog" (textarea 20 40 white))))))))


     (addFileMenuItems (tt "editor-file-menu") win (tt "staprog"))
     (.pack win)
      win))


    (display (list "Net address: " (N 'get 'address))) (newline)
    (.resize mywin 800 400)
    (.show mywin)
    mywin
)



  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; ... start-p2p-chat
  ;; this creates a peer-to-peer chat window with the specified user
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



   (define (start-p2p-chat User/Address)
      (define win 
        (window (string-append (readstring (t "username")) 
                            " is chatting with " User/Address)
        (menubar 
          (menu "File" 
            (menuitem "quit" (action (lambda(e) 
              (.hide win))))))
        (p2p-chat-panel User/Address)))
      (.resize win 200 600)
      (.show win)
    )



  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; ... want-to-chat ...
  ;; ... want-to-help-me ...
  ;; these create various notification windows
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(define (want-to-chat R F)
  (define w (window "CHAT REQUEST" (TimesRomanBold 18)
    (col white
	 (label (.toString (first (third R)))(HelveticaBold 32))
       (label "at IP/port:"(HelveticaBold 32))
       (label (.toString (second (third R)))(HelveticaBold 32))
       (label " would like to chat with you..... " (HelveticaBold 32))
       (row 
         (button "Let's chat!" green (HelveticaBold 32) (action (lambda(e) (F)(.hide w))))
         (button "No thanks..." red (HelveticaBold 32) (action (lambda(e) (.show refusal-win)(.hide w))))))))
  (define refusal-win 
   (let ((ttt (maketagger)))
    (window "CHAT REFUSAL"
      (col
         (label "Your response to the request to chat is"(HelveticaBold 32))
         (ttt "reason" (textfield "no thanks" 40(HelveticaBold 32)))
         (button "send" (HelveticaBold 32)(action (lambda(e)
            (.hide refusal-win)
             (N 'send 
                 (second (third R))
                 'let-us-not-chat
                 (list (readstring (t "username")) 
                       (N 'get 'address))
                 (readstring (ttt "reason")) )
           )))))))
  (.pack w) (.pack refusal-win)
  (.show w))


(define (want-to-help-me R F)
  (define w (window "TA HELP REQUEST" (TimesRomanBold 18) red
    (col white (HelveticaBold 32)
       (label (.toString (first (third R)))(HelveticaBold 32))
       (label "at IP/port:"(HelveticaBold 32))
       (label (.toString (second (third R)))(HelveticaBold 32))
       (label " is asking for help:") (HelveticaBold 32)
       (button "I can help you now" green (HelveticaBold 32)
                 (action (lambda(e) (F)(.hide w))))
       (button "Sorry, I can't help you now..." red (HelveticaBold 32)
                (action (lambda(e) (.show refusal-win)(.hide w)))))))


  (define refusal-win 
   (let ((ttt (maketagger)))
    (window "CHAT REFUSAL"
      (col
         (label "Your negative response to the request to help is")
         (ttt "reason" (textfield "Sorry, I'm busy now.  Please try again later." 40))
         (button "send" (action (lambda(e)
            (.hide refusal-win)
            (N 'send (second (third R)) 'ta-refusal 
                (readstring (t "username"))
                (readstring (ttt "reason")) ))))))))
  (.pack w) (.pack refusal-win)
  (.show w))



  (define (show-message-win Message)
    (define w 
        (window "Message"
           (border
            (center
              (textarea 10 40 Message white (TimesRomanPlain 18)))
            (south (button "OK" red (action (lambda(e) (.hide w))))))))
    (.pack w)
    (.show w)
    w)


(.pack regwin)
(.show regwin)


  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; Now we add listeners for the student (or the ta)
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;; listeners for the registrar ...
(N 'add-listener 'are-you-there (lambda R
  (N 'send REG 'i-am-here name)))

(N 'add-listener 'add-user (lambda R 
    (jlistwrite (t "active-users") 
        (map (lambda(x) (list (first x) (second x))) (second R)))
    (.validate (t "au"))  ))

(N 'add-listener 'drop-user (lambda R 
    (jlistwrite (t "active-users") 
        (map (lambda(x) (list (first x) (second x))) (second R)))
    (.validate (t "au"))  ))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; listeners for messages from other users who want to collaborate
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(N 'add-listener 'let-us-not-chat (lambda R
    (show-message-win (string-append (second R) "\n" (third R) ))))

(N 'add-listener 'maybe-we-can-chat (lambda R
    (show-message-win (string-append (second R) " is thinking about chatting!"))
))

(N 'add-listener 'let-us-not-tat (lambda R
    (show-message-win (string-append (second R) "\n" (third R) ))))
(N 'add-listener 'maybe-we-can-tat (lambda R
    (show-message-win (string-append (second R) " is thinking about tatting!"))))

(N 'add-listener 'let-us-chat 
   (lambda R 
  ;; start a whiteboard
  ;; start a student window     
  (start-p2p-chat  (second R))))

(N 'add-listener 'let-us-tat 
  ;; R is the message, which has the form
  ;; (let-us-tat (their-name (their-host their-port)))
   (lambda R 
     (let ((address (second (second R))))
       (make-super-student name (first (second R)) (first address) (second address) (third R)))))

(N 'add-listener 'chat-refusal (lambda R
  (define w (window "no thanks"
              (border (north (label "Chat Refusal" (HelveticaBold 24)))
                 (center (label (string-append R) (HelveticaBold 18)))
                 (south (row 'none 'center (button "OK" (action (lambda(e) (.hide w)))))))))
  (.pack w) (.show w)))

(N 'add-listener 'ta-refusal (lambda R
  (let ((w (window "sorry"
              (border (north (label "Tat Refusal" (HelveticaBold 24)))
                 (center (label (string-append R) (HelveticaBold 18)))
                 (south (row 'none 'center (button "OK" (action (lambda(e) (.hide w))))))))))
    (.pack w) (.show w))))

(N 'add-listener 'want-to-chat 
   (lambda R 
    (cond 
     ((equal? (readstring (t "autochat")) "accept all chats")
      (N 'send 
          (second (third R))
          'let-us-chat
          (list (readstring (t "username")) 
                (N 'get 'address)))
      (start-p2p-chat (third R)))

     ((equal? (readstring (t "autochat")) "ask me")
      (N 'send 
          (second (third R))
          'maybe-we-can-chat
          (list (readstring (t "username")) 
                (N 'get 'address)))

      (want-to-chat R 
       (lambda ()
          (N 'send 
              (second (third R))
              'let-us-chat
              (list (readstring (t "username")) 
                    (N 'get 'address)))
          (start-p2p-chat (third R)))))
     (else
;      (N 'send (second (third R)) 'chat-refusal 
;           (readstring (t "username"))
;           (readstring (t "refusal message")))

      (N 'send 
          (second (third R))
          'let-us-not-chat
          (list (readstring (t "username")) 
                (N 'get 'address))
          (readstring (t "refusal message")) )

       #t))))

(N 'add-listener 'want-to-help-me
   (lambda R 
      ;; R is the message that got sent of the form:
      ;;   (want-to-help-me groupserver-port (name (host servent-port)))
    (cond 
     ((equal? (readstring (t "autotat")) "accept all tats")
      (N 'send 
          (second (third R))
          'let-us-tat
          (list (readstring (t "username")) 
                (N 'get 'address))
          (list-ref R 3)
          )
       (let ((name (first (third R)))
                (host (first (second (third R))))
                (servent-port (second (second (third R))))
                (groupserver-port (second R))) 
          (make-super-ta name host servent-port groupserver-port (list-ref R 3))
        ))

     ((equal? (readstring (t "autotat")) "ask me")
      (N 'send 
          (second (third R))
          'maybe-we-can-tat
          (list (readstring (t "username")) 
                (N 'get 'address)))

      (want-to-help-me R 
       (lambda ()
          (let ((name (first (third R)))
                (host (first (second (third R))))
                (servent-port (second (second (third R))))
                (groupserver-port (second R))) 
           (N 'send 
              (second (third R))
              'let-us-tat
              (list (readstring (t "username")) 
                    (N 'get 'address))
              (list-ref R 3))
           (make-super-ta name host servent-port groupserver-port))
           )))

     (else
;      (N 'send (second (third R)) 'chat-refusal 
;           (readstring (t "username"))
;           (readstring (t "refusal message")))

      (N 'send 
          (second (third R))
          'let-us-not-tat
          (list (readstring (t "username")) 
                (N 'get 'address))
          (readstring (t "refusal message"))
          )
          ))))

)

(define go start-tatclient)
