;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Util.scm
;;
;; This contains several useful procedures that we couldn't find
;; a better place for and that were not big enough to deserve
;; their own file.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(jlib.Swing.load)

(define remotecolor (java.awt.Color. 200 200 255))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; create a textarea that keeps track of the contents of a shared textarea
;;  (remote-textarea Network Owner TA-Name rows cols ....)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (make-highlighter the-textarea)
  (let ((highlighter (.getHighlighter the-textarea))
        (greenhighlights (javax.swing.text.DefaultHighlighter$DefaultHighlightPainter. green))
        (yellowhighlights (javax.swing.text.DefaultHighlighter$DefaultHighlightPainter. yellow))
       )
  (lambda (offset)
    (.removeAllHighlights highlighter)   
    (let* ((P the-textarea)
           (L (.getLineOfOffset P offset)))
      (.addHighlight highlighter 
          (.getLineStartOffset P  L)
          (.getLineEndOffset P L)
          yellowhighlights))
      (.addHighlight highlighter 
          (max 0 (- offset 100))
          offset
          greenhighlights))))



  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; ... fontmenu  ...
  ;; this creates a fontmenu that sets the fonts for the
  ;; textareas in the list generated by the talistfn
  ;; the argument is a function that returns a list of textareas
  ;; whose fonts should be changed.
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


  (define (fontmenu talistfn)
        (let ((f
               (action (lambda(e) 
                  (let ((f (tryCatch
                              (java.awt.Font. "Courier" java.awt.Font.PLAIN$ 
                                  (java.lang.Integer. (.getActionCommand e)))
                           (lambda (e) (CourierPlain 12)))))
                     (for-each (lambda(x) (.setFont x f)) (talistfn))
                     (for-each (lambda(x) (.validate x)) (talistfn))
                   ))))
              )
         (menu "Font"
            (menuitem "6" f)
            (menuitem "8" f)
            (menuitem "10" f)
            (menuitem "12" f)
            (menuitem "14" f)
            (menuitem "16" f)
            (menuitem "18" f)
            (menuitem "24" f)
            (menuitem "32" f)
         )))








;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;      (maketagger)
;;
;; this creates a tagger, which is a closure that encapsulates a hashtable
;;  (define t (maketagger))  ;; creates the tagger
;;  (t "abc" OBJECT)  ;; stores the object in the hashtable under the key "abc" and returns the OBJECT
;;  (t "abc")         ;; looks up the object in the hashtable stored under the key "abc"
;; This is just a way of providing a local namespace
;; This version of the tagger gives error messages when it is called
;; on non-existent tags...
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define maketagger 
 (lambda()
  (let ((H (java.util.Hashtable.)))
   (lambda R
    (tryCatch
     (cond
       ((equal? (length R) 1)
        (let ((x (.get H (first R))))
           (if (equal? x #null) (begin (display (list "#null tag lookup for " R))(newline)))
           x))
       ((equal? (length R) 2)
        (if (equal? (second R) #null) 
            (begin (display (list "#null value for tagger " R))(newline) #null)
            (.put H (first R) (second R)))
        (second R))
       (else (display (list "wrong number of arguments to tagger" R)) (newline) #null))
     (lambda(e)
         (display (list "ERROR in tagger: args= " R))(newline) #null))
  ))))





