;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; whiteboard.scm
;; Change log:
;;  11/12/2001 TJH
;;    * made cursor affect only top level graphics, not offscreen image
;;    * got text working
;;    * called repaint after all actions
;;  1/26/2002 TJH
;;    * Incorporated whiteboard into grouphw tool
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(jlib.Swing.load)
(import "java.awt.event.MouseEvent")

(define (wbdemo NAME GROUP HOST PORT COLOR)
  (define win (window (string-append NAME "/" GROUP)
     (menubar (menu "File" (menuitem "quit" (action (lambda(e)
      (.hide win) (N 'logout))))))
     (wbpanel  NAME GROUP HOST PORT)))
  win)

(define (wbpanel NAME GROUP HOST PORT COLOR)
 (define jnk1 (begin (display "wb1")(newline)))
 (define N (make-group-client NAME GROUP HOST PORT))
 (define jnk2 (begin (display "wb2")(newline)))

 (define t (maketagger))

 (define hiddenpanel
    (grid 6 1   
	  (row 'none 'center (t "color" (textfield COLOR 15)))
	  (row 'none 'center (t "type" (textfield "Free" 15)))
	  (row 'none 'center (label "Enter text here") )

          (row (t "lastx" (textfield "" 10))  (t "lasty" (textfield "" 10)))))


 (define the-panel
  (border
   (north
     (row (label (string-append "group-server:" HOST "/" PORT))))
   (west
    (grid 1 1

;      (col 'none 'north
;	 (grid 3 1
;	     (button "red" (action (lambda(e) 
;		 (writeexpr (t "color") "red"))))
;	     (button "blue" (action (lambda(e)
;		 (writeexpr (t "color") "blue"))))
;	     (button "green" (action (lambda(e)
;		 (writeexpr (t "color") "green"))))))

	 (col 'none 'north
	(grid 4 1
	      (button "Free" (action (lambda(e) 
		(writeexpr (t "type") "Free"))))
	      (button "Text" (action (lambda(e) 
                (.setEnabled (t "text") #t)
		(writeexpr (t "type") "Text"))))
	      (button "Line" (action (lambda(e)
		 (writeexpr (t "type") "Line"))))
	      (button "Rect" (action (lambda(e)
		 (writeexpr (t "type") "Rect")))))))) 
  (south 
      (t "text" (textfield "" 15 (action (lambda(e) 
             (drawToGraphics (.bufferg$ (t "board")) laststring (readstring (t "text")) (readstring (t "color")) lastx lasty)
             (.repaint (t "board"))

             (set! laststring "")
             (writestring (t "text") "")
             (.setEnabled (t "text") #f)
             (writestring (t "type") "Free"))))))
  (center
   (t "board" (grouptool.tatool.JSchemePanel. 300 350)))  
   ;(t "board" (javax.swing.JPanel.)))
    ))


(define lastx 0)
(define lasty 0)
(define clastx 0)
(define clasty 0)
(define laststring "")

(define (string->color S)
  (case S
   (("red") (color 255 0 0))
   (("green") (color 0 255 0))
   (("blue") (color 0 0 255))))


(define (mouseaction e)
 ;(define g (.getGraphics (t "board")))

 (cond 
   ((equal? (.getID e) MouseEvent.MOUSE_MOVED$)
    (begin
     (N 'send 'abc 'cursor 'nocursor clastx clasty (.getX e) (.getY e) (readstring (t "color")))
     (set! clastx (.getX e))
     (set! clasty (.getY e))))
   ((equal? (.getID e) MouseEvent.MOUSE_ENTERED$)
     (set! clastx (.getX e))
     (set! clasty (.getY e))
     (N 'send 'abc 'startend-cursor 'nocursor (.getX e) (.getY e) (readstring (t "color"))))
   ((equal? (.getID e) MouseEvent.MOUSE_EXITED$)
     (N 'send 'abc 'startend-cursor 'nocursor clastx clasty (readstring (t "color"))))
   )


 (case (readexpr (t "type"))
 ((Text)
      (if (equal? (.getID e) MouseEvent.MOUSE_PRESSED$)
        (begin
          (writestring (t "text") "")
          (set! laststring "")
          (set! lastx (.getX e))
          (set! lasty (.getY e))
          (writeexpr (t "lastx") lastx) 
          (writeexpr (t "lasty") lasty) 
          (.requestFocus (t "text")))))
 ((Free)
       (begin
         (if (equal? (.getID e) MouseEvent.MOUSE_DRAGGED$)
            (begin      
                (N 'send 'abc 'free 'nocursor lastx lasty (.getX e) (.getY e) (readstring (t "color")))
		(.setColor (.getGraphics (t "board")) (string->color (readstring (t "color"))))
		(.drawLine (.getGraphics (t "board")) lastx lasty (.getX e) (.getY e))
		(.setColor (.bufferg$ (t "board")) (string->color (readstring (t "color"))))
		(.drawLine (.bufferg$ (t "board")) lastx lasty (.getX e) (.getY e))
                (.repaint (t "board"))
          ))

         (set! lastx (.getX e))
         (set! lasty (.getY e))
        ))
 ((Line)
       (begin       
        (if (equal? (.getID e) MouseEvent.MOUSE_PRESSED$)
 	  (begin
 	    (set! lastx (.getX e))
 	    (set! lasty (.getY e))))

        (if (equal? (.getID e) MouseEvent.MOUSE_RELEASED$)
 	  (begin
	    (N 'send 'abc 'line 'nocursor lastx lasty (.getX e) (.getY e) (readstring (t "color"))
	       (.setColor (.bufferg$ (t "board")) (string->color (readstring (t "color"))))                   
	       (.drawLine (.bufferg$ (t "board")) lastx lasty (.getX e) (.getY e))
	       (.repaint (t "board")))))
	      
        ))
  ((Rect)
        (if (equal? (.getID e) MouseEvent.MOUSE_PRESSED$)
  	  (begin
  	    (set! lastx (.getX e))
  	    (set! lasty (.getY e))))

        (if (equal? (.getID e) MouseEvent.MOUSE_RELEASED$)
  	  (begin
	    (let ((width (- (.getX e) lastx))
		  (height (- (.getY e) lasty)))
	      (N 'send 'abc 'rect 'nocursor lastx lasty width height (readstring (t "color")))
	      (.setColor (.bufferg$ (t "board")) (string->color (readstring (t "color"))))
	      (.drawRect (.bufferg$ (t "board")) lastx lasty width height)
	      (.repaint (t "board"))
            )
        )))
))

(define (drawToGraphics graphics laststring currentstring color lastx lasty)
    (.setColor graphics white)
    (.drawString graphics laststring lastx lasty)
    (.setColor graphics (string->color color))
    (.drawString graphics  currentstring lastx lasty)
)

(define (draw-string laststring currentstring color lastx lasty)
    (drawToGraphics (.getGraphics (t "board")) laststring currentstring color lastx lasty)
    (drawToGraphics (.bufferg$ (t "board")) laststring currentstring color lastx lasty)
    (.repaint (t "board"))
)

 (define jnk3 (begin (display "wb3")(newline)))

(.setBackground (t "board") white)

(.addMouseMotionListener (t "board")  (silk.Listener. mouseaction))
(.addMouseListener       (t "board")  (silk.Listener. mouseaction))
(.addKeyListener (t "text") (silk.Listener. (lambda(e)
  (let (
	;(g (.getGraphics (t "board")))
        (s (readstring (t "text")))
       )
    (drawToGraphics (.getGraphics (t "board")) laststring s (readstring (t "color")) lastx lasty)
    (writeexpr (t "lastx") lastx)
    (writeexpr (t "lasty") lasty)
    (set! laststring s)
    (N 'send 'abc 'text 'nocursor lastx lasty laststring s (readstring (t "color")))
))))


   ;;; R has form (abc lastx lasty x y)
(N 'add-listener 'abc (lambda R
 (case (second R)
  ((text) 
    (let (
	  ;(g (.getGraphics (t "board")))
          (lastx (list-ref R 3))
          (lasty (list-ref R 4))
          (laststring (list-ref R 5))
          (current (list-ref R 6))
          (color (list-ref R 7))
         )
     (drawToGraphics (.bufferg$ (t "board")) laststring current color lastx lasty) 
     (.repaint (t "board"))
            
   ))

  ((free)
    (let (
	  ;(g (.getGraphics (t "board")))
	  )
      (.setColor (.bufferg$ (t "board")) (string->color (list-ref R 7)))
      (.drawLine (.bufferg$ (t "board"))  (list-ref R 3) (list-ref R 4) (list-ref R 5) (list-ref R 6))
      (.repaint (t "board"))
   ))

  ((line)
   (let (
	 ;(g (.getGraphics (t "board")))
	 )
     (.setColor (.bufferg$ (t "board")) (string->color (list-ref R 7)))
     (.drawLine (.bufferg$ (t "board")) (list-ref R 3) (list-ref R 4) (list-ref R 5) (list-ref R 6))
     (.repaint (t "board"))))

   ((rect)
   (let (
	 ;(g (.getGraphics (t "board")))
	 )
     (.setColor (.bufferg$ (t "board")) (string->color (list-ref R 7)))
     (.drawRect (.bufferg$ (t "board")) (list-ref R 3) (list-ref R 4) (list-ref R 5) (list-ref R 6))
     (.repaint (t "board"))))

   ((cursor)
    (let (
	  (g (.getGraphics (t "board")))
	  )
         (.setXORMode g white)
         (.setColor g(.darker (string->color (list-ref R 7))))
         (.drawOval g (+ -5 (list-ref R 3)) (+ -5 (list-ref R 4)) 10 10)
         (.drawOval g (+ -5 (list-ref R 5)) (+ -5 (list-ref R 6)) 10 10)
	 (.setPaintMode g)))
         

   ((startend-cursor)
    (let (
	  (g (.getGraphics (t "board")))
	  )
         (.setXORMode g white)
         (.setColor g (.darker (string->color (list-ref R 5))))
         (.drawOval g (+ -5 (list-ref R 3)) (+ -5 (list-ref R 4)) 10 10)
         (.setPaintMode g)))



  (else (display (list "we need some integration!!!" R))(newline)))))
(begin (display "wb4")(newline))

the-panel
)

(define (starts)
  (make-group-server 54321))






(define (go who group)
  (wbdemo who group "localhost" 54321))




























