;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  jlibeval.scm -- this is a "modular" implementation of jlib
;;
;;  use:
;;    (jlibeval expr)
;;  the idea is that it evaluates expr using the jlib without
;;  filling up the name space....


(define-macro (class-case varlist . clauses)
   `(cond
      ,@(map 
           (lambda (clause)
             (if (equal? (first clause) 'else)
                 clause  
                `((and 
                    ,@(map (lambda(v c) `(.isInstance ,c ,v)) varlist (first clause)))
                    ,@(rest clause))))
           clauses)))


(define  (jlibevallist exprlist results)
  (let ()
    (define (processConArgs c as) 
        (define (processConArg c a)
          (class-case (c a)
            ((java.awt.Component.class java.lang.String.class)
             (writestring c a))
            ((java.awt.Component.class java.awt.Color.class)
             (.setBackground c a))
            ((java.awt.Component.class silk.Procedure.class)
             (a c))
            ((java.awt.Component.class java.awt.Font.class)
             (.setFont c a))
            ((java.awt.Component.class java.awt.Dimension.class)
             (.setSize c a))
            ((java.awt.Frame.class java.lang.String.class)
             (.setTitle c a))
            ((java.awt.Frame.class java.awt.MenuBar.class)
             (.setMenuBar c a))
            ((java.awt.Container.class java.awt.LayoutManager.class)
             (.setLayoutManager c a))
            ((java.awt.Container.class java.awt.Component.class)
             (.add c a))
            ((jlib.SilkCanvas.class java.awt.Image.class)
           (.paintHandler$ c (lambda(g)        (.setSize c (.getWidth a c) (.getHeight a c))       (.drawImage g a 0 0 c))))
            ((java.awt.Button.class silk.JavaListener.class)
             (.addActionListener c a))
            ((java.awt.TextField.class silk.JavaListener.class)
             (.addActionListener c a))
            ((java.awt.Choice.class silk.JavaListener.class)
             (.addItemListener c a))
            ((java.awt.Checkbox.class silk.JavaListener.class)
             (.addItemListener c a))
            ((java.awt.MenuItem.class silk.JavaListener.class)
             (.addActionListener c a))
            ((java.awt.CheckboxMenuItem.class silk.JavaListener.class)
             (.addItemListener c a))
            ((java.awt.MenuComponent.class java.lang.String.class)
             (writestring c a))
            ((java.awt.MenuComponent.class java.awt.Color.class)
             (.setBackground c a))
            ((java.awt.MenuComponent.class silk.Procedure.class)
             (a c))
            ((java.awt.MenuComponent.class java.awt.Font.class)
             (.setFont c a))
            ((java.awt.MenuBar.class java.awt.Menu.class)
             (.add c a))
            ((java.awt.Menu.class java.awt.MenuComponent.class)
             (.add c a))
            ((java.lang.Object.class java.lang.Object.class)
             (error (list "don't know how to process arg " a (.getClass a) " of component " c (.getClass c))))
           ))
      (for-each (lambda (x) (processConArg c x)) as)
      c
    )
    
    
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Component constructors
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    (define button (lambda (s . R)
      (let ((B (java.awt.Button. s)))
        (processConArgs B R))))
      
    (define choice (lambda R
      (let ((C (java.awt.Choice.)))
        (processConArgs C R))))
      
    (define label (lambda (s . R)
      (let ((L (java.awt.Label. s)))
        (processConArgs L R))))
    
    (define textfield (lambda (s n . R)
      (let ((T (java.awt.TextField. s n)))
      (processConArgs T R))))
    
    (define textarea (lambda (r c . R)
      (let ((T (java.awt.TextArea. r c)))
      (processConArgs T R))))
    
    (define imagecanvas 
      (let* ((c (java.awt.Canvas.))
            (MT (java.awt.MediaTracker. c)))
        (lambda (C Name . R)
          (let ((I (if (.isInstance java.applet.Applet.class C)
                        (.getImage C (.getDocumentBase C) Name)
                        (.getImage (java.awt.Toolkit.getDefaultToolkit) Name))))
            (.addImage MT I 1)
            (.waitForAll MT)
            (let ((T (jlib.SilkCanvas. (.getWidth I c) (.getHeight I c))))
              (.paintHandler$ T (lambda(g) (.drawImage g I 0 0 c) (.paintHandler$ T #null)))
              (processConArgs T R))))))
    
    (define canvas (lambda (r c . R)
      (let ((T (jlib.SilkCanvas. r c)))
      (processConArgs T R))))
    
    (define (image c name)
     (class-case (c name)
       ((java.applet.Applet.class java.lang.String.class)
        (.getImage applet (.getDocumentBase applet) name))
       ((java.awt.Component.class java.lang.String.class)
        (.getImage (java.awt.Toolkit.getDefaultToolkit) name))))
    
    (define window (lambda R
      (let ((win (java.awt.Frame.)))
        (.addWindowListener win (silk.Listener11. (lambda(e) 
           (if (equal? (.getID e) java.awt.event.WindowEvent.WINDOW_CLOSING$)
               (.hide win)))))
        (.setLayout win (java.awt.GridLayout. 1 1))
        (processConArgs win R))))
    
    (define window2 (lambda R
       (display (list "window2 arglist is" R))(newline)
    ))
    
    (define checkboxgroup java.awt.CheckboxGroup.)
    
    (define checkboxmenuitem (lambda R
      (define M (java.awt.CheckboxMenuItem.))
       (processConArgs M R)
       M))
    
    (define menuitem (lambda R
      (define M (java.awt.MenuItem.))
       (processConArgs M R)
       M))
    
    (define menu (lambda R
      (define M (java.awt.Menu. (first R)))
      (processConArgs M (map (lambda (x) (if (.isInstance String.class x) (menuitem x) x))(rest R)))
      M))
    
    (define menubar (lambda R
      (define M (java.awt.MenuBar.))
      (processConArgs M R)
      M))
    
      
    
    
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Part 2. JLIB I/O
    ;;    Generic procedures for reading and writing strings 
    ;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    (define (readstring Comp)
      (class-case (Comp)
         ((java.awt.Button.class) (.getLabel Comp))
         ((java.awt.MenuItem.class) (.getLabel Comp))
         ((java.awt.Choice.class) (.getSelectedItem Comp))
         ((java.awt.List.class) (.getSelectedItem Comp))
         ((java.awt.Label.class) (.getText Comp))
         ((java.awt.TextComponent.class) (.getText Comp))))
    
    
    (define (writeOneString Comp Arg)
      (class-case (Comp)
        ((java.awt.Button.class)   (.setLabel Comp (.toString Arg)))
        ((java.awt.MenuItem.class) (.setLabel Comp (.toString Arg)))
        ((java.awt.Menu.class)     (.add      Comp (.toString Arg)))
        ((java.awt.Choice.class)   (.addItem  Comp (.toString Arg)))
        ((java.awt.List.class)     (.addItem  Comp (.toString Arg)))
        ((java.awt.Label.class)    (.setText  Comp (.toString Arg)))
        ((java.awt.TextComponent.class) (.setText  Comp (.toString Arg)))
      ))
    
    (define writeexpr   (lambda (C . Args) (writeOneString C (apply string-append (map .toString Args)))))
    (define writelnexpr (lambda (C . Args) (writeOneString C (apply string-append (map .toString (append Args (list "\n")))))))
    (define appendexpr   (lambda (Comp . Args) (let  ((T (readstring Comp))) (apply writeexpr (cons Comp (cons T Args))))))
    (define appendlnexpr (lambda (Comp . Args) (apply appendexpr (cons Comp (append Args (list "\n"))))))
    (define readexpr     (lambda (Comp)        (string->expr (readstring Comp))))
    (define readexprlist (lambda (Comp)        (string->exprlist (readstring Comp))))
    (define writestring    writeexpr)
    (define writelnstring  writelnexpr)
    (define appendstring   appendexpr)
    (define appendlnstring appendexpr)
    
    (define expr->string silk.U.stringify)
    (define (string->expr x)  (read  (silk.InputPort.  (java.io.StringReader. x))))
    (define (string->exprlist x)
      (define (readlist port)
        (let ((a (read port)))
           (if (eof-object? a) ()
               (cons a (readlist port)))))
     (readlist (silk.InputPort.  (java.io.StringReader. x))))
    
    
    
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;
    ;;  LAYOUT CONSTRUCTORS
    ;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; border layouts
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (define (center X) (list "Center" X))
    (define (north X)  (list "North" X))
    (define (south X)  (list "South" X))
    (define (east X)   (list "East" X))
    (define (west X)   (list "West" X))
    
    ;;   (border (list "Center" C) (list "North" N) color font ...)
    (define border (lambda R
      (define b (java.awt.Panel.))
      (define (processBorderArgs R)
          (if (pair? (first R))
              (case (first (first R))
                    (("Center" "North" "South" "East" "West") 
                     (.add b (first (first R)) (second (first R))) (processBorderArgs (rest R)))
                    (else (error (list "Expected Center, North, South, East, or West "))))
              (processConArgs b R)))
      (.setLayout b (java.awt.BorderLayout.))
      (processBorderArgs R)
      b))
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; grid constructor
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    (define grid (lambda (r c . R)
      (let ((g (java.awt.Panel.)))
        (.setLayout g (java.awt.GridLayout. r c))
        (processConArgs g R)
    )))
    
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; variations on the grid bag layout constructor
    ;;
    ;; GridBagLayout allows one to use 11 values
    ;; to specify how a component should be placed
    ;; in the container. These values are stored
    ;; as a  GridBagConstraint.
    ;; The  fields are
    ;;   gridx gridy    int or RELATIVE
    ;;   gridwidth gridheight int or REMAINDER
    ;;   fill NONE BOTH HORIZONTAL VERTICAL
    ;;   ipadx ipady  int
    ;;   insets java.awt.Insets
    ;;   anchor CENTER NORTH NORTHEAST ...
    ;;   weightx, weighty double
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    ;; create a grid bag constraint with specified intial values for all fields
    (define (gbc-constraint C gx gy gw gh f ipx ipy Is a wtx wty)
      (.gridx$ C gx)  (.gridy$ C gy)
      (.gridwidth$ C gw) (.gridheight$ C gh)
      (.fill$ C f)
      (.ipadx$ C ipx) (.ipady$ C ipy)
      (.insets$ C Is)
      (.anchor$ C a)
      (.weightx$ C wtx) (.weighty$ C wty)
      C)
    
    
    
    
    
    (define (processGBCarg A C box gbc)
      (class-case (A)
        ((java.awt.Component.class)  (.setConstraints gbc A C) (.add box A))
        ((silk.Symbol.class       )  (processSymbol A C))
        ((java.awt.Color.class    )  (.setBackground box A))
        ((silk.Pair.class         )  (processPair (first A) (second A) C))
        ((java.lang.Object.class  )  (error "unrecognized box element " A))
      ))
    
    
    
    (define (processSymbol S C)
          (case S
            ((both)       (.fill$ C java.awt.GridBagConstraints.BOTH$))
            ((none)       (.fill$ C java.awt.GridBagConstraints.NONE$))
            ((horizontal) (.fill$ C java.awt.GridBagConstraints.HORIZONTAL$))
            ((vertical)   (.fill$ C java.awt.GridBagConstraints.VERTICAL$))
            ((north)      (.anchor$ C java.awt.GridBagConstraints.NORTH$))
            ((northeast)  (.anchor$ C java.awt.GridBagConstraints.NORTHEAST$))
            ((northwest)  (.anchor$ C java.awt.GridBagConstraints.NORTHWEST$))
            ((south)      (.anchor$ C java.awt.GridBagConstraints.SOUTH$))
            ((southeast)  (.anchor$ C java.awt.GridBagConstraints.SOUTHEAST$))
            ((southwest)  (.anchor$ C java.awt.GridBagConstraints.SOUTHWEST$))
            ((center)     (.anchor$ C java.awt.GridBagConstraints.CENTER$))
            ((east)       (.anchor$ C java.awt.GridBagConstraints.EAST$))
            ((west)       (.anchor$ C java.awt.GridBagConstraints.WEST$))
            (else         (error "Unknown box attribute" (.toString S)))))
    
    
    
    (define box (lambda R
      (define box (java.awt.Panel.))
      (define C (java.awt.GridBagConstraints.))
      (define gbc (java.awt.GridBagLayout.))
    
    
      (define (processPair name value C)
          (case name
            ((fill)     (.fill$ C value))
            ((anchor)   (.anchor$ C value))
            ((weightx)  (.weightx$ C value))
            ((weighty)  (.weighty$ C value))
                 (else       (error "unrecognized box element " A))))
    
      (define (processGBCargs Args C box gbc)
        (if (null? Args) box
          (begin
             (processGBCarg (first Args) C box gbc)
             (processGBCargs (rest Args) C box gbc))))
    
      (define (processGBCtableargs type r c cols Args C box gbc)
        (if (equal? type 'cols)
            (begin    (.gridx$ C r)    (.gridy$ C c))
            (begin    (.gridx$ C c)    (.gridy$ C r)))
        (if (null? Args) box
          (let*
               ((A (first Args))
                (newcell (.isInstance java.awt.Component.class A))
                (newcol (and newcell (= c cols)))
                (c1 (if newcol 1 (if newcell (+ 1 c) c)))
                (r1 (if newcol (+ 1 r) r)))
             (processGBCarg (first Args) C box gbc)
             (processGBCtableargs type r1 c1 cols (rest Args) C box gbc))))
    
      (.setLayout box gbc)
      (gbc-constraint C
            (if (equal? (first R) 'row)   java.awt.GridBagConstraints.RELATIVE$ 1)
            (if (equal? (first R) 'row)   1 java.awt.GridBagConstraints.RELATIVE$)
            1 1 
            java.awt.GridBagConstraints.BOTH$ 1 1 
            (java.awt.Insets. 1 1 1 1) 
            java.awt.GridBagConstraints.CENTER$ 1.0 1.0)
       (case (first R)
          ((rows cols)
              (processGBCtableargs (first R) 1 1 (second R) (rest(rest R)) C box gbc))
          ((row col)
              (processGBCargs (rest R) C box gbc))
          (else
              (throw (list "ERROR in call to " `(box ,@R)))))))
    
    (define row (lambda R (apply box (cons 'row R))))
    (define table (lambda (r c . R) (apply box (cons 'rows (cons c R)))))
    (define tablebyrow (lambda R (apply box (cons 'rows R))))
    (define col (lambda R (apply box (cons 'col R))))
    (define tablebycol (lambda R (apply box (cons 'cols R))))
    
    (define (splitrow a1 a2 c1 c2)
      (define p (java.awt.Panel.))
      (define C (java.awt.GridBagConstraints.))
      (define gbc (java.awt.GridBagLayout.))
      (.setLayout p gbc)
      (gbc-constraint C 
            java.awt.GridBagConstraints.RELATIVE$ 1
            1 1 
            java.awt.GridBagConstraints.BOTH$ 1 1 
            (java.awt.Insets. 1 1 1 1) 
            java.awt.GridBagConstraints.CENTER$ (.doubleValue a1) 1.0)
      (.setConstraints gbc c1 C)
      (.add p c1)
      (.weightx$ C (.doubleValue a2))
      (.setConstraints gbc c2 C)
      (.add p c2)
      p
    )
    
    (define (splitcol a1 a2 c1 c2)
      (define p (java.awt.Panel.))
      (define C (java.awt.GridBagConstraints.))
      (define gbc (java.awt.GridBagLayout.))
      (.setLayout p gbc)
      (gbc-constraint C 
            1 java.awt.GridBagConstraints.RELATIVE$
            1 1 
            java.awt.GridBagConstraints.BOTH$ 1 1 
            (java.awt.Insets. 1 1 1 1) 
            java.awt.GridBagConstraints.CENTER$ 1.0 (.doubleValue a1))
      (.setConstraints gbc c1 C)
      (.add p c1)
      (.weighty$ C (.doubleValue a2))
      (.setConstraints gbc c2 C)
      (.add p c2)
      p
    )
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Card layout
    ;;  usage:
    ;; (define mycards 
    ;;   (apply cards (map (lambda(x) (card (.toString x) (button (.toString x) (eval x))))
    ;;                               '(red green yellow blue orange cyan magenta white black))))
    ;; (define win (window "test" (col mycards 
    ;;       (apply row 
    ;;           (map (lambda (x) (button (.toString x) (action (lambda(e) (showcard mycards x))))) 
    ;;                (append '(first last next previous) 
    ;;                         (map .toString '(red green yellow blue orange cyan magenta white black))))))))
    ;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    (define cards (lambda R
      (define data (java.awt.Panel.))
      (define dataManager (java.awt.CardLayout.))
      (.setLayout data dataManager)
      (processConArgs data R)
      data))
    
    (define (card name component)
      (lambda (this) (.add this component name)))
    
    (define (showcard data which)
      (case which
         ((first)    (.first    (.getLayout data) data))
         ((last)     (.last     (.getLayout data) data))
         ((next)     (.next     (.getLayout data) data))
         ((previous) (.previous (.getLayout data) data))
         (else       (.show     (.getLayout data) data which))))
    
    
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Shortcuts -- Fonts and Colors
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    (define (Courier N)       (java.awt.Font. "Courier" java.awt.Font.PLAIN$ N))
    (define (CourierPlain N)  (java.awt.Font. "Courier" java.awt.Font.PLAIN$ N))
    (define (CourierBold N)   (java.awt.Font. "Courier" java.awt.Font.BOLD$ N))
    (define (CourierItalic N) (java.awt.Font. "Courier" java.awt.Font.ITALIC$ N))
    
    (define (TimesRoman N)       (java.awt.Font. "TimesRoman" java.awt.Font.PLAIN$ N))
    (define (TimesRomanPlain N)  (java.awt.Font. "TimesRoman" java.awt.Font.PLAIN$ N))
    (define (TimesRomanBold N)   (java.awt.Font. "TimesRoman" java.awt.Font.BOLD$ N))
    (define (TimesRomanItalic N) (java.awt.Font. "TimesRoman" java.awt.Font.ITALIC$ N))
    
    (define (Helvetica N)       (java.awt.Font. "Helvetica" java.awt.Font.PLAIN$ N))
    (define (HelveticaPlain N)  (java.awt.Font. "Helvetica" java.awt.Font.PLAIN$ N))
    (define (HelveticaBold N)   (java.awt.Font. "Helvetica" java.awt.Font.BOLD$ N))   
    (define (HelveticaItalic N) (java.awt.Font. "Helvetica" java.awt.Font.ITALIC$ N))
    
    ;; color shortcuts
    
    (define white     java.awt.Color.white$)
    (define lightGray java.awt.Color.lightGray$)
    (define gray      java.awt.Color.gray$)
    (define darkGray  java.awt.Color.darkGray$)
    (define black     java.awt.Color.black$)
    (define red       java.awt.Color.red$)
    (define pink      java.awt.Color.pink$)
    (define orange    java.awt.Color.orange$)
    (define yellow    java.awt.Color.yellow$)
    (define green     java.awt.Color.green$)
    (define magenta   java.awt.Color.magenta$)
    (define cyan      java.awt.Color.cyan$)
    (define blue      java.awt.Color.blue$)
    (define color     java.awt.Color.)
    
    (define (size w h) (java.awt.Dimension. w h))
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; tagging -- a simple interface to a hashtable
    ;;   (define tag (maketagger)) -- creates a hashtable associated to tag
    ;;   (tag "abc" OBJECT)  adds "abc"-OBJECT to tag hashtable and returns Object
    ;;   (tag "abc") looks up "abc" in hashtable to get corresponding object
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    (define (maketagger)
      (let ((globals (java.util.Hashtable.)))
        (lambda (T . Vs)
          (let (
            (R (cond 
                ((null? Vs) 
                 (.get globals T))
                ((and (pair? Vs) (null? (cdr Vs)))
                 (.put globals T (first Vs))
                 (first Vs))
                (else (throw `("improper use of \"tag\": " (tag  ,T  . ,Vs)))))))
           (if (eq? R #null)
              (throw (list "undefined tag " (first Vs)))
              R)))))
    
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Signalling Errors in a GUI interaction
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    (define actionerrortextarea (textarea 20 80 white))
    (define actionerrorwindow
      (let ((win (window "ERROR" 
            (border 
               (north (label "ERRORS generated during event handling" green (HelveticaBold 24)))
               (center actionerrortextarea)
               (south (button "clear" (silk.Listener11. (lambda (e) (.setText actionerrortextarea "")))))))))
         (.setBackground win red)
         (.pack win)
         win))
    
    
    (define error (lambda R
      (.println silk.Scheme.error$ (apply string-append (cons "JLIB USAGE ERROR:" (map silk.U.stringify R))))))
    
    
    (define (action p) (silk.Listener11. (lambda (e) 
          (tryCatch (p e) (lambda (ee) 
               (appendlnstring actionerrortextarea 
                   "\n\nERROR in Action in reponse to an AWT event" 
                    e 
                    " is " 
                   ee)
               (.show actionerrorwindow)
               )))))
    
 (define (show x) (display x) (newline) x)

 (define (safe-jlib-eval exprlist)
  (tryCatch 
   (eval
    `(let (
        (jlib.JLIB.load (lambda() #t))
        (button ,button)
        (choice ,choice)
        (label ,label)
        (textfield ,textfield)
        (textarea ,textarea)
        (window ,    window    )
        (checkboxgroup ,checkboxgroup)
        (checkboxmenuitem ,checkboxmenuitem)
        (menuitem ,menuitem)
        (menu ,menu)
        (menubar ,menubar)
        (readstring ,readstring)
        (writeOneString ,writeOneString)
        (writeexpr ,writeexpr)
        (writelnexpr ,writelnexpr)
        (appendexpr ,appendexpr)
        (appendlnexpr ,appendlnexpr)
        (readexpr ,readexpr)
        (readexprlist ,readexprlist)
        (writestring ,writestring)
        (writelnstring ,writelnstring)
        (appendstring ,appendstring)
        (appendlnstring ,appendlnstring)
        (expr->string ,expr->string)
        (string->expr ,string->expr)
        (string->exprlist ,string->exprlist)
        (border ,border)
        (grid ,grid)
        (box ,box)
        (row ,row)
        (col ,col)
        (table ,table)
        (splitrow ,splitrow)
        (splitcol ,splitcol)
        (cards ,cards)
        (card ,card)
        (showcard ,showcard)
        (Courier ,Courier)
        (CourierPlain ,CourierPlain)
        (CourierBold ,CourierBold)
        (CourierItalic ,CourierItalic)
        (TimesRoman ,TimesRoman)
        (TimesRomanPlain ,TimesRomanPlain)
        (TimesRomanBold ,TimesRomanBold)
        (TimesRomanItalic ,TimesRomanItalic)
        (Helvetica ,Helvetica)
        (HelveticaPlain ,HelveticaPlain)
        (HelveticaBold ,HelveticaBold)
        (HelveticaItalic ,HelveticaItalic)
        (white ,white)
        (lightGray ,lightGray)
        (gray ,gray)
        (darkGray ,darkGray)
        (black ,black)
        (red ,red)
        (pink ,pink)
        (orange ,orange)
        (yellow ,yellow)
        (green ,green)
        (magenta ,magenta)
        (cyan ,cyan)
        (blue ,blue)
        (color ,color)
        (size ,size)
        (maketagger ,maketagger)
        (action ,action)
        (error ,error)
      )
    (map (lambda (y z)
      (display (list y ">>" z)) (newline)
      (.append ,results (string-append "\n" y))
      (let ((r z))
        (.append ,results (string-append ">> " r))
        r))
      ',exprlist
      (list ,@exprlist))))
   (lambda(e) (list "ERROR: " e))))

   (set! silk.BacktraceException.printJavaTrace$ #f)
    
     (safe-jlib-eval exprlist)))

    
    
