; idea:
;;  walk through the expression keeping track of the positions of the parent expressions
;;  this doesn't correctly handle all syntax, e.g. #'"' and #'\' and #';'
;;  ideally we should implement this incrementally so that if the user just types one
;;  character at a time one doesn't have to do much recomputation......
;;  only when multiple inserts/deletes are performed or the position is changed.

;; this returns a list (A B)
;; of positions of the beginning and ending of the 
;; minimal S-expression containing the character at position L
(define (current-expr-bounds L text1)
  (define text (string-append text1 ")))))))))))))))))))))))))))"))
  (define S (.length text))
  (define (min2 x y) (cond ((= x -1) y) ((= y -1) x) ((< x y) x) (else y)))
  (define (min3 x y z) (min2 x (min2 y z)))
  (define (skipquote p end)
;     (display (list 'skipquote p))(newline)
     (let* ((m (.indexOf text "\"" p))
            (n (.indexOf text "\\" p))
            (q (min2 m n)))
       (cond ((= q -1) S)
             ((>= q (- S 1)) S)
             ((= q n) (skipquote (+ q 2) S))
             ((= q m) (+ 1 m)))))

  (define (iter pos parents start end)
;    (display (list 'iter start pos end parents (string-append "\n" (U.stringify (string-append "||||" (.substring text 0 pos) "++++++" (.substring text (min pos end) (max pos end)) "||||"))))) (newline)
    (if (>= pos end) 
        (if (null? parents)
            (list S S)
            (iter pos () (first parents) S))
        (let (
              (p (.indexOf text "(" pos))
              (q (.indexOf text ")" pos))
              (s (.indexOf text ";" pos))
              (d (.indexOf text "\"" pos)))
          (let ((m (min2 (min2 p q) (min2 s d))))
;          (display (list "   aaa" p q s d))(newline)
          (if (> m end)
            (if (null? parents)
                (list S S)
                (iter pos () (first parents) S))
            (if (= m -1) 
                (if (null? parents) 
                    (list S S)
                    (list (first parents) S))
                (if (= m q) 
                  (if (null? parents)
                      (list start (+ 1 m))
                      (iter (+ 1 m) (rest parents) start end))
                  (if (= m p) 
                      (iter (+ 1 m) (cons m parents) start end)
                      (if (= m s)  
                          (iter (+ 1 (.indexOf text "\n" m)) parents start end)
                          (iter (skipquote (+ 1 m) end) parents start end))))))))))

  ;; this iterates until it need to the end of the current expression
  (let ((zz  (iter 0 '() 0 L)))
;     (display (list 'answer zz))(newline)(newline)(newline)
     zz)
)



(define (run-editor)
  (define t (textarea 10 60 white))
  (define ans (textarea 10 60 white))
  (define status (textfield "" 60))

  (define evalbutton (button "eval" (action (lambda(e)
   (tryCatch
    (for-each
       (lambda(expr)
         (appendlnexpr ans 
            (string-append "\n>> " expr "\n"
               (tryCatch (eval expr) (lambda(e) e)))))
       (readexprlist t))
    (lambda(e) (appendlnexpr ans (list "parse error: " e))))))))

  (define w (window "editor demo" 
      (border 
        (north evalbutton)
        (center (vsplit 0.5 (scrollpane t) (scrollpane ans))))))

  (define greenhighlights (javax.swing.text.DefaultHighlighter$DefaultHighlightPainter. green))
  (define yellowhighlights (javax.swing.text.DefaultHighlighter$DefaultHighlightPainter. yellow))
  (define redhighlights (javax.swing.text.DefaultHighlighter$DefaultHighlightPainter. red))

  (.pack w) (.show w)
  (.addKeyListener t (Listener.(lambda(e) 
     (let* 
          ((P1 (.getCaretPosition t))
           (P (if (> P1 0) (- P1 1) P1))
           (code (.getText t))
           (Bs (current-expr-bounds P code))
           (endpos (second Bs))
           (startpos (first Bs))
           (program_highlighter (.getHighlighter t)))
     (writeexpr status (list startpos P endpos))
     (.removeAllHighlights program_highlighter)
     (cond 
       ((and (< startpos P) (< P endpos))
        (.addHighlight program_highlighter startpos P yellowhighlights)
        (.addHighlight program_highlighter P  endpos greenhighlights))
       ((< startpos endpos)
        (.addHighlight program_highlighter startpos endpos yellowhighlights))
       ((>= P endpos)
        (.addHighlight program_highlighter endpos P redhighlights))
       (else
        (.addHighlight program_highlighter endpos startpos redhighlights)))))))
)

(jlib.Swing.load)


     
     
