;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; analyze.scm
;; 1/9/2002
;; Tim Hickey
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(jlib.Swing.load)
;; this requires tat/GenericFileMenu.scm and tat/FileMenu.scm
(define maketagger 
  (let ((maketagger maketagger))
    (lambda ()
      (let ((T (maketagger)))
        (lambda R
           (tryCatch (apply T R)
             (lambda (e) (display (list "Error in tagger args " R)) #null)))))))




  (define (r) (load "analyze.scm"))


  (define vcr (maketagger))


  (define (load-log filename)
    (define (read-in in L) 
      (let ((x (read in)))
        (if (eof-object? x) L (read-in in (cons x L)))))
    (define in (open-input-file filename))
    (define L1 (read-in in ()))
    (define L (reverse L1))
    (let ((start (caar L))
          (end (caar L1)))
       (vcr "start" start)
       (vcr "end" end)
       (vcr "log" L)
       L))

  ;; state variables
  (define stop #f)
  (define pause #f)
  (define speedup 1)
  (define direction 'forward)

  ;; vcr view of the tool
  (define remotecolor (color 200 200 255))

  (define vcrscreen (vcr "screen"
   (window 
    (border
   (center
    (vcr "h0"
    (vsplit 200    
     (vcr "h1"
      (hsplit 75
        (border (north (label "student comments")) (center
          (vcr "first" (scrollpane (vcr "stcomm" (textarea 10 20 remotecolor))))))
       (vcr "h2" (hsplit 250
        (border
          (north (label "student program")) 
          (center
            (vcr "second" (scrollpane (vcr "stprog" (textarea 10 20 remotecolor)
                       )))))
        (border (north  (label "ta comments")) (center
          (vcr "third" (scrollpane (vcr "tacomm" (textarea 10 20))))))))))
    (vcr "chats"
     (hsplit 150
        (vcr "wbpanel" (textarea  5 5))
        (scrollpane (vcr "chat-panel" (textarea  10 50))))))))))))



  (define (update-vcr the-time net-type in/out port message)
    (if (and 
          (equal? net-type 'group) 
          (equal? in/out 'in)
          (pair? message)
          (pair? (first message)))
    (let* (
       (x (first message))
       (where (first x))   ;; tacomm, stcomm, or stprog
       (action (second x)) ;; insert, delete
       (what (third x))    ;; shared-textarea
          )
     (case action
       ((insert delete) #t) ;; ignore these

       ((modify-highlights)
        (let* ((data (rest message))
               (insoffset (first data))
               (inscolor (second data))
               (len (third data)))
           (if (equal? inscolor 'green)
		  (begin
		    (set! greenoffset (+ insoffset len))
		    (if (> blueoffset insoffset)
			  (set! blueoffset (+ blueoffset len))))
		  (begin
		    (set! blueoffset (+ insoffset len))
		    (if (> greenoffset insoffset)
			(set! greenoffset (+ greenoffset len)))))
	      (highlight greenoffset blueoffset)))


       ((logger)
         (let* ((data (rest (rest message))) ;; start-point ins/del-text size-of-ins/del 
                (offset (first data))
                (str (second data))
                (len (third data)))
          (case (second message)
            ((insert)
              (display (list '.insertString where offset str))(newline)
              (.insertString (.getDocument (vcr (.toString where))) offset str #null))
            ((delete)
              (display (list '.remove where offset len))(newline)
              (.remove (.getDocument (vcr (.toString where))) offset len))
            (else 
               (display (list "ERROR in update-vcr:unknown action" action the-time net-type in/out port message))
               (newline)))))
       (else (display (list "ERROR in update-vcr:unknown action" action the-time net-type in/out port message))
              (newline)))))


    (if (and 
          (equal? net-type 'servent) 
          (equal? in/out 'in)
          (string? (first message)))
        (appendlnexpr (vcr "chat-panel")
             (string-append (first message) "\n" (second message) "\n")))

    (if (and 
          (equal? net-type 'servent) 
          (equal? in/out 'out)
          (equal? 'send (first message))
          (string? (third message))
         )
        (appendlnexpr (vcr "chat-panel")
             (string-append (first (cddr message)) "\n" (second (cddr message)) "\n")))

    (if (and
          (equal? net-type 'group) 
          (equal? in/out 'in)
          (pair? message)
          (equal? 'abc (first message)))
       (whiteboard-action message))
  )






  (define (OLD-update-vcr the-time net-type in/out port message)
    (if (and 
          (equal? net-type 'group) 
          (equal? in/out 'in)
          (pair? (first message)))
    (let* (
       (x (first message))
       (where (first x))   ;; tacomm, stcomm, or stprog
       (action (second x)) ;; insert, delete
       (what (third x))    ;; shared-textarea
          )
     (case action
       ((insert)
          (.insertString (.getDocument (vcr (.toString where))) (second message) (third message) #null))
       ((delete)
          (.remove (.getDocument (vcr (.toString where))) (second message) (third message)))
       (else (display (list "ERROR in update-vcr:unknown action" action the-time net-type in/out port message))
              (newline)))))


    (if (and 
          (equal? net-type 'servent) 
          (equal? in/out 'in)
          (string? (first message)))
        (appendlnexpr (vcr "chat-panel")
             (string-append (first message) "\n" (second message) "\n")))

    (if (and 
          (equal? net-type 'servent) 
          (equal? in/out 'out)
          (equal? 'send (first message))
          (string? (third message))
         )
        (appendlnexpr (vcr "chat-panel")
             (string-append (first (cddr message)) "\n" (second (cddr message)) "\n")))

  )       


;; Here we construct the main VCR controller window
;;
;; This is a little messy because we want to use a file open/save dialog
;; in the file menu, but this requires a reference to the parent window
;; which is in the process of being created.....
;; None of this would be necessary if we could pass a function returning a
;; parent window into the Dialog window constructor....

 (define (reset-vcr)
 ;; stop all paused or running threads
    (set! stop #t) 
    (java.lang.Thread.sleep 500L) 
 ;; reset stop/pause
    (set! stop #f) 
    (set! pause #f)
    (writeexpr (vcr "pausebutton") "pause")
 ;; reset vcrscreen view
    (map        
      (lambda(x) (writeexpr (vcr x) "")) 
      (list "stcomm" "tacomm" "stprog" "chat-panel")))

 (define vcrwin  
  (let (
     (win  
       (window "VCR" white
        (menubar
          (vcr "filemenu" (menu "File" )) white)
        (border
         (north (vcr "filedata" (label "-----------------------------" white)))
         (south
           (table 4 2
             (vcr "the-time" (label "--------------------------------------------------" white))
             (vcr "speedup" (choice "1" "2" "3" "4" "5" "10" "100" "1000" "1000000" white (action (lambda(e)
                 (set! speedup (readexpr (vcr "speedup")))))))
             (button "start" green (action (lambda(e) 
                          ;; first, we must kill any paused threads! 
                (reset-vcr)
                (.start (java.lang.Thread. (lambda() (play (first (vcr "log")) (rest (vcr "log")))))))))
             
             (button "stop" red (action (lambda(e) (reset-vcr))))

             (button "pause" yellow
               (lambda(this) (vcr "pausebutton" this)) ;; bind "pausebutton" to this button
               (action (lambda(e) 
                (if pause 
                   (begin (writeexpr (vcr "pausebutton") "pause") (set! pause #f))
                   (begin (writeexpr (vcr "pausebutton") "continue") (set! pause #t))))))

             white ;; for table
          ))white  ;; for border
         )))
      (menu-controller
         (lambda (command . Args) 
          (tryCatch
           (case command
             ((string-to-save) (string-append (list (vcr "logfile") "\n" (vcr "bookmarks"))))
             ((parent-win) (vcr "win"))
             ((quit) (.hide vcrwin) (.hide vcrscreen))
             ((new open) 
                 (if (equal? command 'new)
                    (begin
                        (vcr "filename" "default.ana") 
                        (vcr "logfile" (get-new-log-file))
                        (vcr "bookmarks" ()))
                    (begin
                        (vcr "filename" (.getName (first Args)))
                        (vcr "logfile" (first (second Args)))
                        (vcr "bookmarks" (second (second Args)))))
                   (load-log (vcr "logfile")) (display (list "loading log")) (newline)
                   (writeexpr (vcr "filedata") 
                    (string-append (vcr "filename") " / "
                     (* 0.001 (- (vcr "end") (vcr "start"))) " seconds/ "
                     (length (vcr "log")) " events/ "
                     (java.util.Date. (vcr "start")) ": start time"
                    ))
                   ; handle bookmarks ....
                 ))
             (lambda(e) 'abort-menu-action)) ;; this inelegantly handles the case of selecting "cancel"
        ))
     ) ; end of let

   ;; now we add the file menu
    (addFileMenuItems (vcr "filemenu") win menu-controller)
   ;; and return the fully constructed window
    win
  ))


 ;; this lets the user select a new raw log data file
 (define get-new-log-file
  (let ((FileDialogLoad 
        (java.awt.FileDialog. vcrwin "Select Raw Log File" java.awt.FileDialog.LOAD$)))
   (lambda ()
      (.show FileDialogLoad)
      (.getAbsolutePath (java.io.File. (.getDirectory FileDialogLoad) (.getFile FileDialogLoad)))
   )))

    




  (define (play X L)
   (let (
         (the-time (first X))
         (net-type (second X))
         (in/out (third X))
         (port (list-ref X 3))
         (message (list-ref X 4)))

      (if (or (null? L) stop)
          'done
         (if pause
          (let loop() 
              (java.lang.Thread.sleep 100L) 
              (if stop 'done
                 (if pause (loop) 
                    (play X L))))
          (let* (
              (y (first L))
              (y-time (first y))
              (real-delay (- y-time the-time))
              (delay (/ real-delay speedup))
                )
            (update-vcr the-time net-type in/out port message)
            (sleep-and-show-clock the-time real-delay speedup)
            (play y (rest L)))))))


    (define (sleep-and-show-clock the-time real-delay speedup)
       (writelnexpr (vcr "the-time") (java.util.Date. the-time))
       (if (< real-delay 1000) 
         (if (> real-delay 0)
             (java.lang.Thread.sleep (.longValue (/ real-delay speedup)))
             (display (list "SLEEP ERROR: " the-time real-delay speedup)))
         (begin
            (java.lang.Thread.sleep (.longValue (/ 1000 speedup)))
            (sleep-and-show-clock (+ 1000 the-time) (- real-delay 1000) speedup))))
            
      


    (define (listener leader follower1 follower2)
	(Listener. (lambda(e)
		     (.setValue follower1 (.getValue leader))
		     (.setValue follower2  (.getValue leader))
                   )))

    (vcr "first-scroll" (.getVerticalScrollBar (vcr "first")))
    (vcr "second-scroll"(.getVerticalScrollBar (vcr "second")))
    (vcr "third-scroll" (.getVerticalScrollBar (vcr "third")))

    (.addAdjustmentListener (vcr "second-scroll") (listener (vcr "second-scroll") (vcr "second-scroll") (vcr "third-scroll")))
    (.addAdjustmentListener (vcr "first-scroll") (listener (vcr "first-scroll") (vcr "first-scroll") (vcr "third-scroll")))
    (.addAdjustmentListener (vcr "third-scroll") (listener (vcr "third-scroll") (vcr "second-scroll") (vcr "first-scroll")))




   (.pack vcrscreen)
   (.show vcrscreen)

   (.pack vcrwin)
   (.show vcrwin)


